@@ -47,10 +47,10 @@ extern "C" {
 #include <errno.h>
 
 /* set errno to the specified value - then return -1 */
-#define slurm_seterrno_ret(errnum) do { \
-	slurm_seterrno(errnum);         \
-	return (errnum ? -1 : 0);       \
-	} while (0)
+#define slurm_seterrno_ret(errnum) do {	\
+	errno = errnum;			\
+	return (errnum ? -1 : 0);	\
+} while (0)
 
 /* general return codes */
 #define SLURM_SUCCESS   0
@@ -73,6 +73,9 @@ typedef enum {
 	SLURM_PLUGIN_NAME_INVALID,
 	SLURM_UNKNOWN_FORWARD_ADDR,
 	SLURM_COMMUNICATIONS_MISSING_SOCKET_ERROR,
+	SLURM_COMMUNICATIONS_INVALID_INCOMING_FD,
+	SLURM_COMMUNICATIONS_INVALID_OUTGOING_FD,
+	SLURM_COMMUNICATIONS_INVALID_FD,
 
 	/* communication failures to/from slurmctld */
 	SLURMCTLD_COMMUNICATIONS_CONNECTION_ERROR =     1800,
@@ -80,6 +83,7 @@ typedef enum {
 	SLURMCTLD_COMMUNICATIONS_RECEIVE_ERROR,
 	SLURMCTLD_COMMUNICATIONS_SHUTDOWN_ERROR,
 	SLURMCTLD_COMMUNICATIONS_BACKOFF,
+	SLURMCTLD_COMMUNICATIONS_HARD_DROP,
 
 	/* _info.c/communication layer RESPONSE_SLURM_RC message codes */
 	SLURM_NO_CHANGE_IN_DATA =			1900,
@@ -126,8 +130,8 @@ typedef enum {
 	ESLURM_DEPENDENCY,
 	ESLURM_BATCH_ONLY,
 	ESLURM_LICENSES_UNAVAILABLE =			2040,
-
-	ESLURM_JOB_HELD =				2042,
+	ESLURM_TAKEOVER_NO_HEARTBEAT,
+	ESLURM_JOB_HELD,
 	ESLURM_INVALID_CRED_TYPE_CHANGE,
 	ESLURM_INVALID_TASK_MEMORY,
 	ESLURM_INVALID_ACCOUNT,
@@ -148,6 +152,7 @@ typedef enum {
 	ESLURM_PORTS_INVALID =				2060,
 	ESLURM_PROLOG_RUNNING,
 	ESLURM_NO_STEPS,
+	ESLURM_MISSING_WORK_DIR,
 
 	ESLURM_INVALID_QOS =				2066,
 	ESLURM_QOS_PREEMPTION_LOOP,
@@ -187,8 +192,8 @@ typedef enum {
 	ESLURM_BURST_BUFFER_WAIT =			2100,
 	ESLURM_PARTITION_DOWN,
 	ESLURM_DUPLICATE_GRES,
-	ESLURM_JOB_SETTING_DB_INX,
-	ESLURM_RSV_ALREADY_STARTED,
+
+	ESLURM_RSV_ALREADY_STARTED =			2104,
 	ESLURM_SUBMISSIONS_DISABLED,
 	ESLURM_NOT_HET_JOB,
 	ESLURM_NOT_HET_JOB_LEADER,
@@ -246,6 +251,23 @@ typedef enum {
 	ESLURM_INVALID_HET_STEP_JOB,
 	ESLURM_JOB_TIMEOUT_KILLED,
 	ESLURM_JOB_NODE_FAIL_KILLED,
+	ESLURM_EMPTY_LIST,
+	ESLURM_GROUP_ID_INVALID,
+	ESLURM_GROUP_ID_UNKNOWN,
+	ESLURM_USER_ID_INVALID,
+	ESLURM_USER_ID_UNKNOWN,
+	ESLURM_INVALID_ASSOC,
+	ESLURM_NODE_ALREADY_EXISTS,
+	ESLURM_NODE_TABLE_FULL,
+	ESLURM_INVALID_RELATIVE_QOS,
+	ESLURM_INVALID_EXTRA,
+	ESLURM_JOB_SIGNAL_FAILED,
+	ESLURM_SIGNAL_JOBS_INVALID,
+	ESLURM_RES_CORES_PER_GPU_UNIQUE,
+	ESLURM_RES_CORES_PER_GPU_TOPO,
+	ESLURM_RES_CORES_PER_GPU_NO,
+	ESLURM_MAX_POWERED_NODES,
+	ESLURM_REQUESTED_TOPO_CONFIG_UNAVAILABLE,
 
 	/* SPANK errors */
 	ESPANK_ERROR = 					3000,
@@ -286,6 +308,7 @@ typedef enum {
 	ESLURMD_CONTAINER_RUNTIME_INVALID,
 	ESLURMD_CPU_BIND_ERROR,
 	ESLURMD_CPU_LAYOUT_ERROR,
+	ESLURMD_TOO_MANY_RPCS,
 
 	/* socket specific Slurm communications error */
 	ESLURM_PROTOCOL_INCOMPLETE_PACKET = 5003,
@@ -294,6 +317,7 @@ typedef enum {
 
 	/* slurm_auth errors */
 	ESLURM_AUTH_CRED_INVALID	= 6000,
+	ESLURM_AUTH_EXPIRED,
 	ESLURM_AUTH_BADARG		= 6004,
 	ESLURM_AUTH_UNPACK		= 6007,
 	ESLURM_AUTH_SKIP,
@@ -310,6 +334,9 @@ typedef enum {
 	ESLURM_DB_QUERY_TOO_WIDE,
 	ESLURM_DB_CONNECTION_INVALID,
 	ESLURM_NO_REMOVE_DEFAULT_ACCOUNT,
+	ESLURM_BAD_SQL,
+	ESLURM_NO_REMOVE_DEFAULT_QOS,
+	ESLURM_COORD_NO_INCREASE_JOB_LIMIT,
 
 	/* Federation Errors */
 	ESLURM_FED_CLUSTER_MAX_CNT              = 7100,
@@ -326,6 +353,12 @@ typedef enum {
 	ESLURM_PLUGIN_INVALID,
 	ESLURM_PLUGIN_INCOMPLETE,
 	ESLURM_PLUGIN_NOT_LOADED,
+	ESLURM_PLUGIN_NOTFOUND,
+	ESLURM_PLUGIN_ACCESS_ERROR,
+	ESLURM_PLUGIN_DLOPEN_FAILED,
+	ESLURM_PLUGIN_INIT_FAILED,
+	ESLURM_PLUGIN_MISSING_NAME,
+	ESLURM_PLUGIN_BAD_VERSION,
 
 	/* REST errors */
 	ESLURM_REST_INVALID_QUERY = 9000,
@@ -349,6 +382,9 @@ typedef enum {
 	ESLURM_DATA_AMBIGUOUS_MODIFY,
 	ESLURM_DATA_AMBIGUOUS_QUERY,
 	ESLURM_DATA_PARSE_NOTHING,
+	ESLURM_DATA_INVALID_PARSER,
+	ESLURM_DATA_PARSING_DEPTH,
+	ESLURM_DATA_PARSER_INVALID_STATE,
 
 	/* container errors */
 	ESLURM_CONTAINER_NOT_CONFIGURED = 10000,
@@ -370,9 +406,6 @@ char * slurm_strerror(int errnum);
 /* set an errno value */
 void slurm_seterrno(int errnum);
 
-/* get an errno value */
-int slurm_get_errno(void);
-
 /* print message: error string for current errno value */
 void slurm_perror(const char *msg);
 
