@@ -100,10 +100,10 @@ typedef struct {
 	char *cmdstr;
 	int *fildes;
 	int mode;
+	bool perform_mount;
 	int rlimit;
 	char **tmp_env;
 	const char *username;
-	bool setup_namespaces;
 } child_args_t;
 
 /*
@@ -425,8 +425,6 @@ int setup_env(env_t *env, bool preserve_env)
 
 		if (env->cpu_bind_type & CPU_BIND_NONE) {
 			str_bind2 = "none";
-		} else if (env->cpu_bind_type & CPU_BIND_RANK) {
-			str_bind2 = "rank";
 		} else if (env->cpu_bind_type & CPU_BIND_MAP) {
 			str_bind2 = "map_cpu:";
 		} else if (env->cpu_bind_type & CPU_BIND_MASK) {
@@ -621,6 +619,12 @@ int setup_env(env_t *env, bool preserve_env)
 		rc = SLURM_ERROR;
 	}
 
+	if (env->oom_kill_step != NO_VAL16 &&
+	    setenvf(&env->env, "SLURM_OOM_KILL_STEP", "%u", env->oom_kill_step)) {
+		error("Unable to set SLURM_OOM_KILL_STEP environment");
+		rc = SLURM_ERROR;
+	}
+
 	if (env->slurmd_debug
 	    && setenvf(&env->env, "SLURMD_DEBUG", "%d", env->slurmd_debug)) {
 		error("Can't set SLURMD_DEBUG environment variable");
@@ -2095,19 +2099,21 @@ static int _child_fn(void *arg)
 	cmdstr = child_args->cmdstr;
 	tmp_env = child_args->tmp_env;
 
+#if !defined(__APPLE__) && !defined(__FreeBSD__) && !defined(__NetBSD__)
 	/*
 	 * Setting propagation and mounting our own /proc for this namespace.
 	 * This is done to ensure that this cloned process and its children
 	 * have coherent /proc contents with their virtual PIDs.
 	 * Check _clone_env_child to see namespace flags used in clone.
 	 */
-	if (child_args->setup_namespaces) {
+	if (child_args->perform_mount) {
 		if (mount("none", "/proc", NULL, MS_PRIVATE|MS_REC, NULL))
 			_exit(1);
 		if (mount("proc", "/proc", "proc",
 			  MS_NOSUID|MS_NOEXEC|MS_NODEV, NULL))
 			_exit(1);
 	}
+#endif
 
 	if ((devnull = open("/dev/null", O_RDWR)) != -1) {
 		dup2(devnull, STDIN_FILENO);
@@ -2140,6 +2146,7 @@ static int _child_fn(void *arg)
 static int _clone_env_child(child_args_t *child_args)
 {
 	char *child_stack;
+	int rc = 0;
 	child_stack = mmap(NULL, STACK_SIZE, PROT_READ | PROT_WRITE,
 			   MAP_PRIVATE | MAP_ANONYMOUS | MAP_STACK, -1, 0);
 	if (child_stack == MAP_FAILED) {
@@ -2155,9 +2162,70 @@ static int _clone_env_child(child_args_t *child_args)
 	 * Killing the 'child' pid will kill all the namespace, since in the
 	 * namespace, this 'child' is pid 1.
 	 */
-	child_args->setup_namespaces = true;
-	return clone(_child_fn, child_stack + STACK_SIZE,
-		     (SIGCHLD|CLONE_NEWPID|CLONE_NEWNS), child_args);
+	rc = clone(_child_fn, child_stack + STACK_SIZE,
+		   (SIGCHLD|CLONE_NEWPID|CLONE_NEWNS), child_args);
+	/* Memory deallocated only in parent address space, child unaffected */
+	if (munmap(child_stack, STACK_SIZE))
+		error("%s: failed to munmap child stack: %m", __func__);
+	return rc;
+}
+
+static bool _ns_path_disabled(const char *ns_path)
+{
+	FILE *fp = NULL;
+	size_t line_sz = 0;
+	ssize_t nbytes = 0;
+	int ns_value;
+	char *line = NULL;
+	bool ns_disabled = false;
+
+	/* We will assume not having these files as having no limits. */
+	fp = fopen(ns_path, "r");
+	if (!fp) {
+		debug2("%s: could not open %s, assuming no pid namespace limits. Reason: %m",
+		       __func__, ns_path);
+	} else {
+		nbytes = getline(&line, &line_sz, fp);
+		if (nbytes < 0) {
+			debug2("%s: could not read contents of %s. Assuming no namespace limits. Reason: %m",
+			       __func__, ns_path);
+		} else if (nbytes == 0) {
+			debug2("%s: read 0 bytes from %s. Assuming no namespace limits",
+			       __func__, ns_path);
+		} else {
+			ns_value = xstrntol(line, NULL, nbytes, 10);
+			if (ns_value == 0)
+				ns_disabled = true;
+		}
+		fclose(fp);
+		free(line);
+		line = NULL;
+	}
+
+	return ns_disabled;
+}
+
+/*
+ * Returns a boolean indicating if the required namespaces for the clone
+ * calls are disabled. This is performed by checking the contents of
+ * "/proc/sys/max_[mnt|pid]_namespaces" and ensuring they are not 0.
+ */
+static bool _ns_disabled()
+{
+	static int disabled = -1;
+	char *pid_ns_path = "/proc/sys/user/max_pid_namespaces";
+	char *mnt_ns_path = "/proc/sys/user/max_mnt_namespaces";
+
+	if (disabled != -1)
+		return disabled;
+
+	disabled = false;
+
+	if (_ns_path_disabled(pid_ns_path) ||
+	    _ns_path_disabled(mnt_ns_path))
+		disabled = true;
+
+	return disabled;
 }
 #endif
 
@@ -2233,8 +2301,8 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 	child_args.fildes = fildes;
 	child_args.username = username;
 	child_args.cmdstr = cmdstr;
-	child_args.setup_namespaces = false;
 	child_args.tmp_env = env_array_create();
+	child_args.perform_mount = true;
 	env_array_overwrite(&child_args.tmp_env, "ENVIRONMENT", "BATCH");
 	if (getrlimit(RLIMIT_NOFILE, &rlim) < 0) {
 		error("getrlimit(RLIMIT_NOFILE): %m");
@@ -2251,9 +2319,27 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 	if (child == 0)
 		_child_fn(&child_args);
 #else
-	if ((child = _clone_env_child(&child_args)) == -1) {
-		fatal("clone: %m");
-		return NULL;
+	/*
+	 * Since we will be using namespaces in the clone calls (CLONE_NEWPID,
+	 * CLONE_NEWNS), we need to know if they are disabled . If they are,
+	 * we must fall back to fork and warn the user about the risks.
+	 */
+	if (_ns_disabled()) {
+		warning("%s: pid or mnt namespaces are disabled, avoiding clone and falling back to fork. This can produce orphan/unconstrained processes!",
+			__func__);
+		child_args.perform_mount = false;
+		child = fork();
+		if (child == -1) {
+			fatal("fork: %m");
+			return NULL;
+		}
+		if (child == 0)
+			_child_fn(&child_args);
+	} else {
+		if ((child = _clone_env_child(&child_args)) == -1) {
+			fatal("clone: %m");
+			return NULL;
+		}
 	}
 #endif
 	close(fildes[1]);
