@@ -65,6 +65,7 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <sys/un.h>
 #include <unistd.h>
 
 #include "slurm/slurm_errno.h"
@@ -144,7 +145,7 @@ char *slurm_prog_name = NULL;
 static pthread_mutex_t  log_lock = PTHREAD_MUTEX_INITIALIZER;
 static log_t            *log = NULL;
 static log_t            *sched_log = NULL;
-
+static bool syslog_open = false;
 static volatile log_level_t highest_log_level = LOG_LEVEL_END;
 static volatile log_level_t highest_sched_log_level = LOG_LEVEL_QUIET;
 
@@ -345,8 +346,16 @@ _log_init(char *prog, log_options_t opt, log_facility_t fac, char *logfile )
 		log->fbuf = cbuf_create(128, 8192);
 	}
 
-	if (log->opt.syslog_level > LOG_LEVEL_QUIET)
+	if (syslog_open) {
+		closelog();
+		syslog_open = false;
+	}
+
+	if (log->opt.syslog_level > LOG_LEVEL_QUIET) {
 		log->facility = fac;
+		openlog(log->argv0, LOG_PID, log->facility);
+		syslog_open = true;
+	}
 
 	if (logfile && (log->opt.logfile_level > LOG_LEVEL_QUIET)) {
 		int mode = O_CREAT | O_WRONLY | O_APPEND | O_CLOEXEC;
@@ -527,6 +536,10 @@ void log_fini(void)
 		cbuf_destroy(log->fbuf);
 	if (log->logfp)
 		fclose(log->logfp);
+	if (syslog_open) {
+		closelog();
+		syslog_open = false;
+	}
 	xfree(log);
 	xfree(slurm_prog_name);
 	slurm_mutex_unlock(&log_lock);
@@ -706,10 +719,6 @@ static void _set_idbuf(char *idbuf, size_t size)
  */
 static char *_addr2fmt(slurm_addr_t *addr_ptr, char *buf, int buf_size)
 {
-	struct sockaddr_in *in = (struct sockaddr_in *) addr_ptr;
-	char addrbuf[INET6_ADDRSTRLEN];
-	uint16_t port = 0;
-
 	/*
 	 * NOTE: You will notice we put a %.0s in front of the string.
 	 * This is to handle the fact that we can't remove the addr_ptr
@@ -720,17 +729,32 @@ static char *_addr2fmt(slurm_addr_t *addr_ptr, char *buf, int buf_size)
 		return "%.0sNULL";
 
 	if (addr_ptr->ss_family == AF_INET6) {
+		char addrbuf[INET6_ADDRSTRLEN];
+		uint16_t port = 0;
 		struct sockaddr_in6 *in6 = (struct sockaddr_in6 *) addr_ptr;
 		inet_ntop(AF_INET6, &in6->sin6_addr, addrbuf, INET6_ADDRSTRLEN);
 		port = ntohs(in6->sin6_port);
 		snprintf(buf, buf_size, "[%%.0s%s]:%d", addrbuf, port);
-		return buf;
+	} else if (addr_ptr->ss_family == AF_INET) {
+		char addrbuf[INET_ADDRSTRLEN];
+		struct sockaddr_in *in = (struct sockaddr_in *) addr_ptr;
+		uint16_t port = 0;
+		inet_ntop(AF_INET, &in->sin_addr, addrbuf, INET_ADDRSTRLEN);
+		port = ntohs(in->sin_port);
+		snprintf(buf, buf_size, "%%.0s%s:%d", addrbuf, port);
+	} else if (addr_ptr->ss_family == AF_UNIX) {
+		struct sockaddr_un *un = (struct sockaddr_un *) addr_ptr;
+		snprintf(buf, buf_size, "%%.0sunix:%s", un->sun_path);
+	} else if (addr_ptr->ss_family == AF_UNSPEC) {
+		/*
+		 * AF_UNSPEC is place holder for unspecified which may be
+		 * nothing or used as a wild card depending on the API call.
+		 */
+		snprintf(buf, buf_size, "%%.0sAF_UNSPEC");
+	} else {
+		snprintf(buf, buf_size, "%%.0sINVALID");
 	}
 
-	inet_ntop(AF_INET, &in->sin_addr, addrbuf, INET_ADDRSTRLEN);
-	port = ntohs(in->sin_port);
-	snprintf(buf, buf_size, "%%.0s%s:%d", addrbuf, port);
-
 	return buf;
 }
 /*
@@ -788,11 +812,11 @@ static char *_stepid2fmt(step_record_t *step_ptr, char *buf, int buf_size)
 				     STEP_ID_FLAG_SPACE | STEP_ID_FLAG_NO_JOB);
 }
 
-static char *_print_data_t(const data_t *d, char *buffer, uint16_t size)
+static char *_print_data_t(const data_t *d, char *buffer, int size)
 {
 	/*
 	 * NOTE: You will notice we put a %.0s in front of the string.
-	 * This is to handle the fact that we can't remove the job_ptr
+	 * This is to handle the fact that we can't remove the data_t
 	 * argument from the va_list directly. So when we call vsnprintf()
 	 * to handle the va_list this will effectively skip this argument.
 	 */
@@ -801,9 +825,30 @@ static char *_print_data_t(const data_t *d, char *buffer, uint16_t size)
 	return buffer;
 }
 
+static char *_print_data_json(const data_t *d, char *buffer, int size)
+{
+	char *nbuf = NULL;
+
+	/*
+	 * NOTE: You will notice we put a %.0s in front of the string.
+	 * This is to handle the fact that we can't remove the data_t
+	 * argument from the va_list directly. So when we call vsnprintf()
+	 * to handle the va_list this will effectively skip this argument.
+	 */
+	if (serialize_g_data_to_string(&nbuf, NULL, d, MIME_TYPE_JSON,
+				       SER_FLAGS_COMPACT))
+		snprintf(buffer, size, "%%.0s(JSON serialization failed)");
+	else
+		snprintf(buffer, size, "%%.0s%s", nbuf);
+
+	xfree(nbuf);
+
+	return buffer;
+}
+
 extern char *vxstrfmt(const char *fmt, va_list ap)
 {
-	char	*intermediate_fmt = NULL;
+	char *intermediate_fmt = NULL, *intermediate_pos = NULL;
 	char	*out_string = NULL;
 	char	*p;
 	bool found_other_formats = false;
@@ -818,7 +863,7 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 			 * no more format sequences, append the rest of
 			 * fmt and exit the loop:
 			 */
-			xstrcat(intermediate_fmt, fmt);
+			xstrcatat(intermediate_fmt, &intermediate_pos, fmt);
 			break;
 		}
 
@@ -837,6 +882,7 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 			case 'p':
 				switch (*(p + 2)) {
 				case 'A':
+				case 'd':
 				case 'D':
 				case 'J':
 				case 's':
@@ -872,7 +918,8 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 			 * append anything from fmt up to p to the intermediate
 			 * format string:
 			 */
-			xstrncat(intermediate_fmt, fmt, p - fmt);
+			xstrncatat(intermediate_fmt, &intermediate_pos,
+				   fmt, p - fmt);
 			fmt = p + 1;
 
 			/*
@@ -893,7 +940,9 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 					for (int i = 0; i < cnt; i++ )
 						ptr = va_arg(ap_copy, void *);
 					addr_ptr = ptr;
-					xstrcat(intermediate_fmt,
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
 						_addr2fmt(
 							addr_ptr,
 							substitute_on_stack,
@@ -901,6 +950,24 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 					va_end(ap_copy);
 					break;
 				}
+				case 'd':	/* "%pd" -> compact JSON serialized string */
+				{
+					data_t *d = NULL;
+					va_list	ap_copy;
+
+					va_copy(ap_copy, ap);
+					for (int i = 0; i < cnt; i++ )
+						d = va_arg(ap_copy, void *);
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
+						_print_data_json(
+							d,
+							substitute_on_stack,
+							sizeof(substitute_on_stack)));
+					va_end(ap_copy);
+					break;
+				}
 				case 'D':	/* "%pD" -> data_type(0xDEADBEEF) */
 				{
 					data_t *d = NULL;
@@ -909,7 +976,9 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 					va_copy(ap_copy, ap);
 					for (int i = 0; i < cnt; i++ )
 						d = va_arg(ap_copy, void *);
-					xstrcat(intermediate_fmt,
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
 						_print_data_t(
 							d,
 							substitute_on_stack,
@@ -928,7 +997,9 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 					for (i = 0; i < cnt; i++ )
 						ptr = va_arg(ap_copy, void *);
 					job_ptr = ptr;
-					xstrcat(intermediate_fmt,
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
 						_jobid2fmt(
 							job_ptr,
 							substitute_on_stack,
@@ -951,7 +1022,9 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 					for (i = 0; i < cnt; i++ )
 						ptr = va_arg(ap_copy, void *);
 					step_id = ptr;
-					xstrcat(intermediate_fmt,
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
 						log_build_step_id_str(
 							step_id,
 							substitute_on_stack,
@@ -979,12 +1052,16 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 					if (step_ptr &&
 					    (step_ptr->magic == STEP_MAGIC))
 						job_ptr = step_ptr->job_ptr;
-					xstrcat(intermediate_fmt,
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
 						_jobid2fmt(
 							job_ptr,
 							substitute_on_stack,
 							sizeof(substitute_on_stack)));
-					xstrcat(intermediate_fmt,
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
 						_stepid2fmt(
 							step_ptr,
 							substitute_on_stack,
@@ -1069,13 +1146,16 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 
 				while (*s && (p = (char *)strchr(s, '%'))) {
 					/* append up through the '%' */
-					xstrncat(intermediate_fmt, s, p - s);
-					xstrcat(intermediate_fmt, "%%");
+					xstrncatat(intermediate_fmt,
+						   &intermediate_pos, s, p - s);
+					xstrcatat(intermediate_fmt,
+						  &intermediate_pos, "%%");
 					s = p + 1;
 				}
 				if (*s) {
 					/* append whatever's left of the substitution: */
-					xstrcat(intermediate_fmt, s);
+					xstrcatat(intermediate_fmt,
+						  &intermediate_pos, s);
 				}
 
 				/* deallocate substitute if necessary: */
@@ -1088,7 +1168,7 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 			 * no more format sequences for us, append the rest of
 			 * fmt and exit the loop:
 			 */
-			xstrcat(intermediate_fmt, fmt);
+			xstrcatat(intermediate_fmt, &intermediate_pos, fmt);
 			break;
 		}
 	}
@@ -1222,6 +1302,13 @@ static void _log_msg(log_level_t level, bool sched, bool spank, bool warn,
 	char *eol = "\n";
 	int priority = LOG_INFO;
 
+	/*
+	 * Construct the message outside the lock as this can be slow.
+	 * The format_print() macro should ensure that we're always going
+	 * to print the resulting message through one or more channels.
+	 */
+	buf = vxstrfmt(fmt, args);
+
 	slurm_mutex_lock(&log_lock);
 
 	if (!LOG_INITIALIZED) {
@@ -1234,7 +1321,6 @@ static void _log_msg(log_level_t level, bool sched, bool spank, bool warn,
 
 	if (SCHED_LOG_INITIALIZED && sched &&
 	    (highest_sched_log_level > LOG_LEVEL_QUIET)) {
-		buf = vxstrfmt(fmt, args);
 		xlogfmtcat(&msgbuf, "[%M] %s%s", sched_log->prefix, pfx);
 		_log_printf(sched_log, sched_log->fbuf, sched_log->logfp,
 			    "sched: %s%s\n", msgbuf, buf);
@@ -1301,12 +1387,6 @@ static void _log_msg(log_level_t level, bool sched, bool spank, bool warn,
 
 	}
 
-	if (!buf) {
-		/* format the basic message,
-		 * if not already done for scheduling log */
-		buf = vxstrfmt(fmt, args);
-	}
-
 	if (level <= log->opt.stderr_level) {
 
 		fflush(stdout);
@@ -1380,12 +1460,10 @@ static void _log_msg(log_level_t level, bool sched, bool spank, bool warn,
 	if (level <=  log->opt.syslog_level) {
 
 		/* Avoid changing errno if syslog fails */
-		int orig_errno = slurm_get_errno();
+		int orig_errno = errno;
 		xlogfmtcat(&msgbuf, "%s%s%s", log->prefix, pfx, buf);
-		openlog(log->argv0, LOG_PID, log->facility);
 		syslog(priority, "%.500s", msgbuf);
-		closelog();
-		slurm_seterrno(orig_errno);
+		errno = orig_errno;
 
 		xfree(msgbuf);
 	}
@@ -1636,7 +1714,8 @@ extern char *log_build_step_id_str(
 	return buf;
 }
 
-extern void _log_flag_hex(const void *data, size_t len, const char *fmt, ...)
+extern void _log_flag_hex(const void *data, size_t len, ssize_t start,
+			  ssize_t end, const char *fmt, ...)
 {
 	va_list ap;
 	char *prepend;
@@ -1645,17 +1724,22 @@ extern void _log_flag_hex(const void *data, size_t len, const char *fmt, ...)
 	if (!data || !len)
 		return;
 
+	if (start < 0)
+		start = 0;
+	if ((end < 0) || (end > len))
+		end = len;
+
 	va_start(ap, fmt);
 	prepend = vxstrfmt(fmt, ap);
 	va_end(ap);
 
-	for (int i = 0; (i < len); ) {
-		int remain = len - i;
+	for (size_t i = start; (i < end); ) {
+		int remain = end - i;
 		int print = (remain < hex_cols) ? remain : hex_cols;
 		char *phex = xstring_bytes2hex((data + i), print, " ");
 		char *pstr = xstring_bytes2printable((data + i), print, '.');
 
-		format_print(LOG_LEVEL_VERBOSE, "%s [%04d/%04zu] 0x%s \"%s\"",
+		format_print(LOG_LEVEL_VERBOSE, "%s [%04zu/%04zu] 0x%s \"%s\"",
 			     prepend, i, len, phex, pstr);
 
 		i += print;
