@@ -2,7 +2,7 @@
  *  proc_args.c - helper functions for command argument processing
  *****************************************************************************
  *  Copyright (C) 2007 Hewlett-Packard Development Company, L.P.
- *  Portions Copyright (C) 2010-2015 SchedMD LLC <https://www.schedmd.com>.
+ *  Copyright (C) SchedMD LLC.
  *  Written by Christopher Holmes <cholmes@hp.com>, who borrowed heavily
  *  from existing Slurm source code, particularly src/srun/opt.c
  *
@@ -519,10 +519,10 @@ bool verify_node_count(const char *arg, int *min_nodes, int *max_nodes,
 		while (tok) {
 			char *endptr;
 			long int num = strtol(tok, &endptr, 10);
-			if ((endptr == tok) || ((*endptr != '\0') &&
-						(*endptr != ',') &&
-						(*endptr != '-') &&
-						(*endptr != ':'))) {
+			if ((endptr == tok) ||
+			    ((*endptr != '\0') && (*endptr != ',') &&
+			     (*endptr != '-') && (*endptr != ':')) ||
+			    (num >= MAX_JOB_SIZE_BITMAP)) {
 				error("\"%s\" is not a valid node count", tok);
 				xfree(tmp_str);
 				return false;
@@ -965,10 +965,9 @@ char *print_mail_type(const uint16_t type)
 	return buf;
 }
 
-static List
-_create_path_list(void)
+static list_t *_create_path_list(void)
 {
-	List l = list_create(xfree_ptr);
+	list_t *l = list_create(xfree_ptr);
 	char *path;
 	char *c, *lc;
 
@@ -1047,8 +1046,8 @@ static bool _accessible(const char *path, int access_mode)
 char *search_path(char *cwd, char *cmd, bool check_cwd_last, int access_mode,
 		  bool test_exec)
 {
-	List         l        = NULL;
-	ListIterator i        = NULL;
+	list_t *l = NULL;
+	list_itr_t *i = NULL;
 	char *path, *fullpath = NULL;
 
 	/* Relative path */
@@ -1591,6 +1590,12 @@ extern uint64_t parse_resv_flags(const char *flagstr, const char *msg,
 					 MAX(taglen, 1)) && op != RESV_REM) {
 			curr += taglen;
 			outflags |= RESERVE_FLAG_NO_HOLD_JOBS;
+		} else if (!xstrncasecmp(curr, "User_Delete", MAX(taglen, 1))) {
+			curr += taglen;
+			if (op == RESV_REM)
+				outflags |= RESERVE_FLAG_NO_USER_DEL;
+			else
+				outflags |= RESERVE_FLAG_USER_DEL;
 		} else {
 			error("Error parsing flags %s.  %s", flagstr, msg);
 			return INFINITE64;
