@@ -56,6 +56,250 @@ typedef struct {
 	char *user_name;
 } add_acct_cond_t;
 
+typedef struct {
+	char *acct;
+	list_t *acct_list;
+	char *cluster_name;
+	slurmdb_assoc_flags_t flags;
+	mysql_conn_t *mysql_conn;
+	char *query;
+	char *query_pos;
+	list_t *user_list;
+} flag_coord_acct_t;
+
+static int _foreach_flag_coord_handle(void *x, void *arg)
+{
+	slurmdb_user_rec_t *user_rec = x;
+	flag_coord_acct_t *flag_coord_acct = arg;
+
+	as_mysql_user_handle_user_coord_flag(
+		user_rec,
+		flag_coord_acct->flags,
+		flag_coord_acct->acct);
+	return 0;
+}
+
+static int _foreach_flag_coord_user(void *x, void *arg)
+{
+	slurmdb_assoc_rec_t *assoc_ptr = x;
+	flag_coord_acct_t *flag_coord_acct = arg;
+	int rc = 0;
+
+	/* In the children_list the user assocs are always first */
+	if (assoc_ptr->user) {
+		slurmdb_user_rec_t *user_rec = as_mysql_user_add_coord_update(
+			flag_coord_acct->mysql_conn,
+			&flag_coord_acct->user_list,
+			assoc_ptr->user,
+			true);
+		as_mysql_user_handle_user_coord_flag(
+			user_rec,
+			flag_coord_acct->flags,
+			assoc_ptr->acct);
+		return 0;
+	}
+
+	/*
+	 * We have a non-user assoc, so add/remove that from the full user list.
+	 */
+	if (flag_coord_acct->user_list) {
+		flag_coord_acct->acct = assoc_ptr->acct;
+		rc = list_for_each(flag_coord_acct->user_list,
+				   _foreach_flag_coord_handle,
+				   flag_coord_acct);
+		flag_coord_acct->acct = NULL;
+	}
+
+	if (assoc_ptr->usage->children_list)
+		rc = list_for_each(assoc_ptr->usage->children_list,
+				   _foreach_flag_coord_user,
+				   flag_coord_acct);
+
+	return rc;
+}
+
+static int _foreach_flag_coord_acct(void *x, void *arg)
+{
+	int rc = 1;
+	flag_coord_acct_t *flag_coord_acct = arg;
+	slurmdb_assoc_rec_t *assoc_ptr = NULL;
+	slurmdb_assoc_rec_t assoc_req = {
+		.cluster = flag_coord_acct->cluster_name,
+		.acct = x,
+		.uid = NO_VAL,
+	};
+
+	if (assoc_mgr_fill_in_assoc(flag_coord_acct->mysql_conn,
+				    &assoc_req,
+				    ACCOUNTING_ENFORCE_ASSOCS,
+				    &assoc_ptr,
+				    true) != SLURM_SUCCESS)
+		return -1;
+	/* Only change if needed */
+	if (((assoc_ptr->flags & ASSOC_FLAG_USER_COORD) &&
+	     (flag_coord_acct->flags & ASSOC_FLAG_USER_COORD_NO)) ||
+	    (!(assoc_ptr->flags & ASSOC_FLAG_USER_COORD) &&
+	     (flag_coord_acct->flags & ASSOC_FLAG_USER_COORD))) {
+		slurmdb_assoc_rec_t *mod_assoc =
+			xmalloc(sizeof(slurmdb_assoc_rec_t));
+		slurmdb_init_assoc_rec(mod_assoc, 0);
+		mod_assoc->id = assoc_ptr->id;
+		mod_assoc->cluster = xstrdup(assoc_ptr->cluster);
+		mod_assoc->flags = assoc_ptr->flags;
+		if (flag_coord_acct->flags & ASSOC_FLAG_USER_COORD_NO)
+			mod_assoc->flags &= ~ASSOC_FLAG_USER_COORD;
+		else
+			mod_assoc->flags |= ASSOC_FLAG_USER_COORD;
+
+		if (addto_update_list(flag_coord_acct->mysql_conn->update_list,
+				      SLURMDB_MODIFY_ASSOC, mod_assoc) !=
+		    SLURM_SUCCESS) {
+			error("Couldn't add removal of coord, this should never happen.");
+			slurmdb_destroy_user_rec(mod_assoc);
+			return 0;
+		}
+
+		/* set up query to remove the flag */
+		if (!flag_coord_acct->query) {
+			xstrfmtcatat(flag_coord_acct->query,
+				     &flag_coord_acct->query_pos,
+				     "update \"%s_%s\" set flags = %u where id_assoc IN (%u",
+				     mod_assoc->cluster, assoc_table,
+				     mod_assoc->flags, mod_assoc->id);
+		} else {
+			xstrfmtcatat(flag_coord_acct->query,
+				     &flag_coord_acct->query_pos,
+				     ",%u",
+				     mod_assoc->id);
+		}
+
+		if (assoc_ptr->usage->children_list)
+			rc = list_for_each(assoc_ptr->usage->children_list,
+					   _foreach_flag_coord_user,
+					   flag_coord_acct);
+	}
+
+	return rc;
+}
+
+static int _foreach_flag_coord_cluster(void *x, void *arg)
+{
+	int rc;
+	flag_coord_acct_t *flag_coord_acct = arg;
+
+	flag_coord_acct->cluster_name = x;
+
+	rc = list_for_each_ro(flag_coord_acct->acct_list,
+			      _foreach_flag_coord_acct,
+			      flag_coord_acct);
+	if (!rc)
+		return rc;
+
+	if (flag_coord_acct->query) {
+		xstrcatat(flag_coord_acct->query,
+			  &flag_coord_acct->query_pos,
+			  ");");
+		/* Now clear the flag for the associations in the database */
+		DB_DEBUG(DB_ASSOC, flag_coord_acct->mysql_conn->conn,
+			 "query\n%s",
+			 flag_coord_acct->query);
+		if ((rc = mysql_db_query(flag_coord_acct->mysql_conn,
+					 flag_coord_acct->query)) !=
+		    SLURM_SUCCESS) {
+			error("Couldn't update flags");
+			rc = 0;
+		}
+
+		xfree(flag_coord_acct->query);
+	}
+
+	return rc;
+}
+
+static void _handle_flag_coord(flag_coord_acct_t *flag_coord_acct)
+{
+	assoc_mgr_lock_t locks = {
+		.assoc = READ_LOCK,
+		.user = READ_LOCK,
+	};
+
+	assoc_mgr_lock(&locks);
+	(void) list_for_each_ro(as_mysql_cluster_list,
+				_foreach_flag_coord_cluster,
+				flag_coord_acct);
+	assoc_mgr_unlock(&locks);
+
+	FREE_NULL_LIST(flag_coord_acct->user_list);
+	xfree(flag_coord_acct->query);
+}
+
+static void _setup_acct_cond_limits(slurmdb_account_cond_t *acct_cond,
+				    char **extra, char **at)
+{
+	list_itr_t *itr = NULL;
+	char *object = NULL;
+
+	if (!acct_cond)
+		return;
+
+	if (acct_cond->assoc_cond &&
+	    acct_cond->assoc_cond->acct_list &&
+	    list_count(acct_cond->assoc_cond->acct_list)) {
+		int set = 0;
+		xstrcatat(*extra, at, " && (");
+		itr = list_iterator_create(acct_cond->assoc_cond->acct_list);
+		while ((object = list_next(itr))) {
+			if (set)
+				xstrcatat(*extra, at, " || ");
+			xstrfmtcatat(*extra, at, "name='%s'", object);
+			set = 1;
+		}
+		list_iterator_destroy(itr);
+		xstrcat(*extra, ")");
+	}
+
+	if (acct_cond->description_list
+	    && list_count(acct_cond->description_list)) {
+		int set = 0;
+		xstrcatat(*extra, at, " && (");
+		itr = list_iterator_create(acct_cond->description_list);
+		while ((object = list_next(itr))) {
+			if (set)
+				xstrcatat(*extra, at, " || ");
+			xstrfmtcatat(*extra, at, "description='%s'", object);
+			set = 1;
+		}
+		list_iterator_destroy(itr);
+		xstrcatat(*extra, at, ")");
+	}
+
+	if (acct_cond->flags != SLURMDB_ACCT_FLAG_NONE) {
+		if (acct_cond->flags & SLURMDB_ACCT_FLAG_USER_COORD_NO) {
+			xstrfmtcatat(*extra, at, " && !(flags & %u)",
+				     SLURMDB_ACCT_FLAG_USER_COORD);
+		} else if (acct_cond->flags & SLURMDB_ACCT_FLAG_USER_COORD) {
+			xstrfmtcatat(*extra, at, " && (flags & %u)",
+				     SLURMDB_ACCT_FLAG_USER_COORD);
+		}
+	}
+
+	if (acct_cond->organization_list
+	    && list_count(acct_cond->organization_list)) {
+		int set = 0;
+		xstrcatat(*extra, at, " && (");
+		itr = list_iterator_create(acct_cond->organization_list);
+		while ((object = list_next(itr))) {
+			if (set)
+				xstrcatat(*extra, at, " || ");
+			xstrfmtcatat(*extra, at, "organization='%s'", object);
+			set = 1;
+		}
+		list_iterator_destroy(itr);
+		xstrcatat(*extra, at, ")");
+	}
+
+	return;
+}
 
 static int _foreach_add_acct(void *x, void *arg)
 {
@@ -69,6 +313,7 @@ static int _foreach_add_acct(void *x, void *arg)
 	MYSQL_RES *result = NULL;
 	int cnt;
 	char *query;
+	slurmdb_acct_flags_t base_flags;
 
 	/* Check to see if it is already in the acct_table */
 	query = xstrdup_printf("select name from %s where name='%s' and !deleted",
@@ -98,6 +343,9 @@ static int _foreach_add_acct(void *x, void *arg)
 			org = x;
 	}
 
+	/* Clear flags we don't plan to store */
+	base_flags = acct->flags & ~SLURMDB_ACCT_FLAG_BASE;
+
 	if (!add_acct_cond->ret_str)
 		xstrcatat(add_acct_cond->ret_str, &add_acct_cond->ret_str_pos,
 			  " Adding Account(s)\n");
@@ -108,18 +356,18 @@ static int _foreach_add_acct(void *x, void *arg)
 	if (add_acct_cond->insert_query)
 		xstrfmtcatat(add_acct_cond->insert_query,
 			     &add_acct_cond->insert_query_pos,
-			     ", (%ld, %ld, '%s', '%s', '%s')",
+			     ", (%ld, %ld, '%s', '%s', '%s', %u)",
 			     add_acct_cond->now, add_acct_cond->now,
-			     name, desc, org);
+			     name, desc, org, base_flags);
 	else
 		xstrfmtcatat(add_acct_cond->insert_query,
 			     &add_acct_cond->insert_query_pos,
-			     "insert into %s (creation_time, mod_time, name, description, organization) values (%ld, %ld, '%s', '%s', '%s')",
+			     "insert into %s (creation_time, mod_time, name, description, organization, flags) values (%ld, %ld, '%s', '%s', '%s', %u)",
 			     acct_table, add_acct_cond->now, add_acct_cond->now,
-			     name, desc, org);
+			     name, desc, org, base_flags);
 
-	extra = xstrdup_printf("description='%s', organization='%s'",
-			       desc, org);
+	extra = xstrdup_printf("description='%s', organization='%s', flags='%u'",
+			       desc, org, base_flags);
 	tmp_extra = slurm_add_slash_to_quotes(extra);
 
 	if (add_acct_cond->txn_query)
@@ -142,9 +390,9 @@ static int _foreach_add_acct(void *x, void *arg)
 }
 
 extern int as_mysql_add_accts(mysql_conn_t *mysql_conn, uint32_t uid,
-			      List acct_list)
+			      list_t *acct_list)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	int rc = SLURM_SUCCESS;
 	slurmdb_account_rec_t *object = NULL;
 	char *cols = NULL, *vals = NULL, *query = NULL, *txn_query = NULL;
@@ -153,7 +401,7 @@ extern int as_mysql_add_accts(mysql_conn_t *mysql_conn, uint32_t uid,
 	char *extra = NULL, *tmp_extra = NULL;
 
 	int affect_rows = 0;
-	List assoc_list;
+	list_t *assoc_list;
 
 	if (check_connection(mysql_conn) != SLURM_SUCCESS)
 		return ESLURM_DB_CONNECTION;
@@ -161,6 +409,11 @@ extern int as_mysql_add_accts(mysql_conn_t *mysql_conn, uint32_t uid,
 	if (!is_user_min_admin_level(mysql_conn, uid, SLURMDB_ADMIN_OPERATOR)) {
 		slurmdb_user_rec_t user;
 
+		if (slurmdbd_conf->flags & DBD_CONF_FLAG_DISABLE_COORD_DBD) {
+			error("Coordinator privilege revoked with DisableCoordDBD, only admins/operators can add accounts.");
+			return ESLURM_ACCESS_DENIED;
+		}
+
 		memset(&user, 0, sizeof(slurmdb_user_rec_t));
 		user.uid = uid;
 
@@ -185,6 +438,8 @@ extern int as_mysql_add_accts(mysql_conn_t *mysql_conn, uint32_t uid,
 	user_name = uid_to_string((uid_t) uid);
 	itr = list_iterator_create(acct_list);
 	while ((object = list_next(itr))) {
+		slurmdb_acct_flags_t base_flags;
+
 		if (!object->name || !object->name[0]
 		    || !object->description || !object->description[0]
 		    || !object->organization || !object->organization[0]) {
@@ -195,13 +450,18 @@ extern int as_mysql_add_accts(mysql_conn_t *mysql_conn, uint32_t uid,
 			rc = SLURM_ERROR;
 			continue;
 		}
+
+		base_flags = object->flags & ~SLURMDB_ACCT_FLAG_BASE;
+
 		xstrcat(cols, "creation_time, mod_time, name, "
-			"description, organization");
-		xstrfmtcat(vals, "%ld, %ld, '%s', '%s', '%s'",
+			"description, organization, flags");
+		xstrfmtcat(vals, "%ld, %ld, '%s', '%s', '%s', %u",
 			   now, now, object->name,
-			   object->description, object->organization);
-		xstrfmtcat(extra, ", description='%s', organization='%s'",
-			   object->description, object->organization);
+			   object->description, object->organization,
+			   base_flags);
+		xstrfmtcat(extra, ", description='%s', organization='%s', flags=%u",
+			   object->description, object->organization,
+			   base_flags);
 
 		query = xstrdup_printf(
 			"insert into %s (%s) values (%s) "
@@ -305,6 +565,13 @@ extern char *as_mysql_add_accts_cond(mysql_conn_t *mysql_conn, uint32_t uid,
 	if (!is_user_min_admin_level(mysql_conn, uid, SLURMDB_ADMIN_OPERATOR)) {
 		slurmdb_user_rec_t user;
 
+		if (slurmdbd_conf->flags & DBD_CONF_FLAG_DISABLE_COORD_DBD) {
+			char *ret_str = xstrdup("Coordinator privilege revoked with DisableCoordDBD, only admins/operators can add accounts.");
+			error("%s", ret_str);
+			errno = ESLURM_ACCESS_DENIED;
+			return ret_str;
+		}
+
 		memset(&user, 0, sizeof(slurmdb_user_rec_t));
 		user.uid = uid;
 
@@ -322,6 +589,10 @@ extern char *as_mysql_add_accts_cond(mysql_conn_t *mysql_conn, uint32_t uid,
 		 */
 	}
 
+	/* Transfer over relavant flags from the account to the association. */
+	if (acct->flags & SLURMDB_ACCT_FLAG_USER_COORD)
+		add_assoc->assoc.flags |= ASSOC_FLAG_USER_COORD;
+
 	memset(&add_acct_cond, 0, sizeof(add_acct_cond));
 	add_acct_cond.acct_in = acct;
 	add_acct_cond.assoc_in = &add_assoc->assoc;
@@ -410,18 +681,17 @@ end_it:
 	return add_acct_cond.ret_str;
 }
 
-extern List as_mysql_modify_accts(mysql_conn_t *mysql_conn, uint32_t uid,
-				  slurmdb_account_cond_t *acct_cond,
-				  slurmdb_account_rec_t *acct)
+extern list_t *as_mysql_modify_accts(mysql_conn_t *mysql_conn, uint32_t uid,
+				     slurmdb_account_cond_t *acct_cond,
+				     slurmdb_account_rec_t *acct)
 {
-	ListIterator itr = NULL;
-	List ret_list = NULL;
+	list_t *ret_list = NULL;
 	int rc = SLURM_SUCCESS;
-	char *object = NULL;
+	char *object = NULL, *at = NULL;
 	char *vals = NULL, *extra = NULL, *query = NULL, *name_char = NULL;
 	time_t now = time(NULL);
 	char *user_name = NULL;
-	int set = 0;
+	slurmdb_assoc_flags_t assoc_flags = ASSOC_FLAG_NONE;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
 
@@ -438,58 +708,24 @@ extern List as_mysql_modify_accts(mysql_conn_t *mysql_conn, uint32_t uid,
 		return NULL;
 	}
 
-	xstrcat(extra, "where deleted=0");
-	if (acct_cond->assoc_cond
-	    && acct_cond->assoc_cond->acct_list
-	    && list_count(acct_cond->assoc_cond->acct_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(acct_cond->assoc_cond->acct_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "name='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
-
-	if (acct_cond->description_list
-	    && list_count(acct_cond->description_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(acct_cond->description_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "description='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
-
-	if (acct_cond->organization_list
-	    && list_count(acct_cond->organization_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(acct_cond->organization_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "organization='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
+	xstrcatat(extra, &at, "where deleted=0");
+	_setup_acct_cond_limits(acct_cond, &extra, &at);
 
 	if (acct->description)
 		xstrfmtcat(vals, ", description='%s'", acct->description);
 	if (acct->organization)
 		xstrfmtcat(vals, ", organization='%s'", acct->organization);
 
+	if (acct->flags & SLURMDB_ACCT_FLAG_USER_COORD_NO) {
+		xstrfmtcat(vals, ", flags=flags&~%u",
+			   SLURMDB_ACCT_FLAG_USER_COORD);
+		assoc_flags |= ASSOC_FLAG_USER_COORD_NO;
+	} else if (acct->flags & SLURMDB_ACCT_FLAG_USER_COORD) {
+		xstrfmtcat(vals, ", flags=flags|%u",
+			   SLURMDB_ACCT_FLAG_USER_COORD);
+		assoc_flags |= ASSOC_FLAG_USER_COORD;
+	}
+
 	if (!extra || !vals) {
 		errno = SLURM_NO_CHANGE_IN_DATA;
 		error("Nothing to change");
@@ -546,24 +782,36 @@ extern List as_mysql_modify_accts(mysql_conn_t *mysql_conn, uint32_t uid,
 	xfree(name_char);
 	xfree(vals);
 
+	if (ret_list &&
+	    (assoc_flags &
+	     (ASSOC_FLAG_USER_COORD_NO | ASSOC_FLAG_USER_COORD))) {
+		flag_coord_acct_t flag_coord_acct = {
+			.acct_list = ret_list,
+			.flags = assoc_flags,
+			.mysql_conn = mysql_conn,
+		};
+
+		/* Update associations based on account flags */
+		_handle_flag_coord(&flag_coord_acct);
+	}
+
 	return ret_list;
 }
 
-extern List as_mysql_remove_accts(mysql_conn_t *mysql_conn, uint32_t uid,
-				  slurmdb_account_cond_t *acct_cond)
+extern list_t *as_mysql_remove_accts(mysql_conn_t *mysql_conn, uint32_t uid,
+				     slurmdb_account_cond_t *acct_cond)
 {
-	ListIterator itr = NULL;
-	List ret_list = NULL;
-	List coord_list = NULL;
-	List cluster_list_tmp = NULL;
+	list_itr_t *itr = NULL;
+	list_t *ret_list = NULL;
+	list_t *coord_list = NULL;
+	list_t *cluster_list_tmp = NULL;
 	int rc = SLURM_SUCCESS;
-	char *object = NULL;
+	char *object = NULL, *at = NULL;
 	char *extra = NULL, *query = NULL,
 		*name_char = NULL, *name_char_pos = NULL,
 		*assoc_char = NULL, *assoc_char_pos = NULL;
 	time_t now = time(NULL);
 	char *user_name = NULL;
-	int set = 0;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
 	bool jobs_running = 0;
@@ -582,54 +830,8 @@ extern List as_mysql_remove_accts(mysql_conn_t *mysql_conn, uint32_t uid,
 		return NULL;
 	}
 
-	xstrcat(extra, "where deleted=0");
-	if (acct_cond->assoc_cond
-	    && acct_cond->assoc_cond->acct_list
-	    && list_count(acct_cond->assoc_cond->acct_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(acct_cond->assoc_cond->acct_list);
-		while ((object = list_next(itr))) {
-			if (!object[0])
-				continue;
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "name='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
-
-	if (acct_cond->description_list
-	    && list_count(acct_cond->description_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(acct_cond->description_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "description='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
-
-	if (acct_cond->organization_list
-	    && list_count(acct_cond->organization_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(acct_cond->organization_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "organization='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
+	xstrcatat(extra, &at, "where deleted=0");
+	_setup_acct_cond_limits(acct_cond, &extra, &at);
 
 	if (!extra) {
 		error("Nothing to remove");
@@ -713,15 +915,14 @@ extern List as_mysql_remove_accts(mysql_conn_t *mysql_conn, uint32_t uid,
 	return ret_list;
 }
 
-extern List as_mysql_get_accts(mysql_conn_t *mysql_conn, uid_t uid,
-			       slurmdb_account_cond_t *acct_cond)
+extern list_t *as_mysql_get_accts(mysql_conn_t *mysql_conn, uid_t uid,
+				  slurmdb_account_cond_t *acct_cond)
 {
 	char *query = NULL;
-	char *extra = NULL;
+	char *extra = NULL, *at = NULL;
 	char *tmp = NULL;
-	List acct_list = NULL;
-	ListIterator itr = NULL;
-	char *object = NULL;
+	list_t *acct_list = NULL;
+	list_itr_t *itr = NULL;
 	int set = 0;
 	int i=0, is_admin=1;
 	MYSQL_RES *result = NULL;
@@ -734,12 +935,14 @@ extern List as_mysql_get_accts(mysql_conn_t *mysql_conn, uid_t uid,
 		"description",
 		"organization",
 		"deleted",
+		"flags",
 	};
 	enum {
 		SLURMDB_REQ_NAME,
 		SLURMDB_REQ_DESC,
 		SLURMDB_REQ_ORG,
 		SLURMDB_REQ_DELETED,
+		SLURMDB_REQ_FLAGS,
 		SLURMDB_REQ_COUNT
 	};
 
@@ -752,6 +955,12 @@ extern List as_mysql_get_accts(mysql_conn_t *mysql_conn, uid_t uid,
 	if (slurm_conf.private_data & PRIVATE_DATA_ACCOUNTS) {
 		if (!(is_admin = is_user_min_admin_level(
 			      mysql_conn, uid, SLURMDB_ADMIN_OPERATOR))) {
+			if (slurmdbd_conf->flags &
+			    DBD_CONF_FLAG_DISABLE_COORD_DBD) {
+				error("Coordinator privilege revoked with DisableCoordDBD, only admins/operators can add accounts.");
+				errno = ESLURM_ACCESS_DENIED;
+				return NULL;
+			}
 			if (!is_user_any_coord(mysql_conn, &user)) {
 				error("Only admins/coordinators "
 				      "can look at account usage");
@@ -766,56 +975,12 @@ extern List as_mysql_get_accts(mysql_conn_t *mysql_conn, uid_t uid,
 		goto empty;
 	}
 
-	if (acct_cond->with_deleted)
-		xstrcat(extra, "where (deleted=0 || deleted=1)");
+	if (acct_cond->flags & SLURMDB_ACCT_FLAG_DELETED)
+		xstrcatat(extra, &at, "where (deleted=0 || deleted=1)");
 	else
-		xstrcat(extra, "where deleted=0");
+		xstrcatat(extra, &at, "where deleted=0");
 
-	if (acct_cond->assoc_cond
-	    && acct_cond->assoc_cond->acct_list
-	    && list_count(acct_cond->assoc_cond->acct_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(acct_cond->assoc_cond->acct_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "name='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
-
-	if (acct_cond->description_list
-	    && list_count(acct_cond->description_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(acct_cond->description_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "description='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
-
-	if (acct_cond->organization_list
-	    && list_count(acct_cond->organization_list)) {
-		set = 0;
-		xstrcat(extra, " && (");
-		itr = list_iterator_create(acct_cond->organization_list);
-		while ((object = list_next(itr))) {
-			if (set)
-				xstrcat(extra, " || ");
-			xstrfmtcat(extra, "organization='%s'", object);
-			set = 1;
-		}
-		list_iterator_destroy(itr);
-		xstrcat(extra, ")");
-	}
+	_setup_acct_cond_limits(acct_cond, &extra, &at);
 
 empty:
 
@@ -862,14 +1027,17 @@ empty:
 
 	acct_list = list_create(slurmdb_destroy_account_rec);
 
-	if (acct_cond && acct_cond->assoc_cond && acct_cond->with_assocs) {
+	if (acct_cond && acct_cond->assoc_cond &&
+	    (acct_cond->flags & SLURMDB_ACCT_FLAG_WASSOC)) {
 		/* We are going to be freeing the inners of
 		   this list in the acct->name so we don't
 		   free it here
 		*/
 		FREE_NULL_LIST(acct_cond->assoc_cond->acct_list);
 		acct_cond->assoc_cond->acct_list = list_create(NULL);
-		acct_cond->assoc_cond->with_deleted = acct_cond->with_deleted;
+		if (acct_cond->flags & SLURMDB_ACCT_FLAG_DELETED)
+			acct_cond->assoc_cond->flags |=
+				ASSOC_COND_FLAG_WITH_DELETED;
 	}
 
 	while ((row = mysql_fetch_row(result))) {
@@ -880,15 +1048,17 @@ empty:
 		acct->name =  xstrdup(row[SLURMDB_REQ_NAME]);
 		acct->description = xstrdup(row[SLURMDB_REQ_DESC]);
 		acct->organization = xstrdup(row[SLURMDB_REQ_ORG]);
+		acct->flags = slurm_atoul(row[SLURMDB_REQ_FLAGS]);
 
 		if (slurm_atoul(row[SLURMDB_REQ_DELETED]))
 			acct->flags |= SLURMDB_ACCT_FLAG_DELETED;
 
-		if (acct_cond && acct_cond->with_coords)
+		if (acct_cond && (acct_cond->flags & SLURMDB_ACCT_FLAG_WCOORD))
 			acct->coordinators =
 				assoc_mgr_acct_coords(mysql_conn, acct->name);
 
-		if (acct_cond && acct_cond->with_assocs) {
+		if (acct_cond && (acct_cond->flags &
+				  SLURMDB_ACCT_FLAG_WASSOC)) {
 			if (!acct_cond->assoc_cond) {
 				acct_cond->assoc_cond = xmalloc(
 					sizeof(slurmdb_assoc_cond_t));
@@ -900,12 +1070,14 @@ empty:
 	}
 	mysql_free_result(result);
 
-	if (acct_cond && acct_cond->with_assocs && acct_cond->assoc_cond
+	if (acct_cond &&
+	    (acct_cond->flags & SLURMDB_ACCT_FLAG_WASSOC) &&
+	    acct_cond->assoc_cond
 	    && list_count(acct_cond->assoc_cond->acct_list)) {
-		ListIterator assoc_itr = NULL;
+		list_itr_t *assoc_itr = NULL;
 		slurmdb_account_rec_t *acct = NULL;
 		slurmdb_assoc_rec_t *assoc = NULL;
-		List assoc_list = as_mysql_get_assocs(
+		list_t *assoc_list = as_mysql_get_assocs(
 			mysql_conn, uid, acct_cond->assoc_cond);
 
 		if (!assoc_list) {
