@@ -58,7 +58,7 @@ typedef struct {
 	ssize_t index;
 	args_t *args;
 	const parser_t *const parser;
-	List list;
+	list_t *list;
 	data_t *dlist;
 	data_t *parent_path;
 } foreach_list_t;
@@ -195,6 +195,7 @@ static data_for_each_cmd_t _foreach_flag_parser(data_t *src, void *arg)
 	const parser_t *const parser = args->parser;
 	char *path = NULL;
 	bool matched_any = false;
+	data_for_each_cmd_t rc = DATA_FOR_EACH_CONT;
 
 	xassert(args->magic == MAGIC_FOREACH_LIST_FLAG);
 	xassert(args->args->magic == MAGIC_ARGS);
@@ -202,6 +203,13 @@ static data_for_each_cmd_t _foreach_flag_parser(data_t *src, void *arg)
 
 	path = _flag_parent_path(&path, args);
 
+	if (data_convert_type(src, DATA_TYPE_STRING) != DATA_TYPE_STRING) {
+		on_warn(PARSING, parser->type, args->args, path,
+			__func__, "Ignoring unexpected field of type %s",
+			data_get_type_string(src));
+		goto cleanup;
+	}
+
 	for (int8_t i = 0; (i < parser->flag_bit_array_count); i++) {
 		const flag_bit_t *bit = &parser->flag_bit_array[i];
 		bool matched = !xstrcasecmp(data_get_string(src), bit->name);
@@ -220,23 +228,26 @@ static data_for_each_cmd_t _foreach_flag_parser(data_t *src, void *arg)
 				_set_flag_bit_equal(parser, dst, bit, matched,
 						    path, src);
 			args->set |= bit->mask;
+		} else if (bit->type == FLAG_BIT_TYPE_REMOVED) {
+			if (matched)
+				on_warn(PARSING, parser->type, args->args, path,
+					__func__,
+					"Ignoring deprecated flag: %s",
+					bit->name);
 		} else
 			fatal_abort("%s: invalid bit_flag_t", __func__);
 
 	}
 
-	args->index++;
-
-	if (!matched_any) {
+	if (!matched_any)
 		on_error(PARSING, parser->type, args->args,
 			 ESLURM_DATA_FLAGS_INVALID, path, __func__,
 			 "Unknown flag \"%s\"", data_get_string(src));
-		xfree(path);
-		return DATA_FOR_EACH_FAIL;
-	}
 
+cleanup:
 	xfree(path);
-	return DATA_FOR_EACH_CONT;
+	args->index++;
+	return rc;
 }
 
 static int _parse_flag(void *dst, const parser_t *const parser, data_t *src,
@@ -342,7 +353,7 @@ static int _parse_list(const parser_t *const parser, void *dst, data_t *src,
 {
 	int rc = SLURM_SUCCESS;
 	char *path = NULL;
-	List *list = dst;
+	list_t **list = dst;
 	foreach_list_t list_args = {
 		.magic = MAGIC_FOREACH_LIST,
 		.dlist = NULL,
@@ -446,6 +457,8 @@ static data_for_each_cmd_t _foreach_array_entry(data_t *src, void *arg)
 	foreach_nt_array_t *args = arg;
 	void *obj = NULL;
 	data_t *ppath = NULL;
+	const parser_t *const parser = args->parser;
+	const parser_t *const array_parser = args->array_parser;
 
 	xassert(args->magic == MAGIC_FOREACH_NT_ARRAY);
 	xassert((args->index > 0) || (args->index == -1));
@@ -464,21 +477,22 @@ static data_for_each_cmd_t _foreach_array_entry(data_t *src, void *arg)
 				    data_get_string(ppath_last), args->index);
 	}
 
-	if (args->parser->model == PARSER_MODEL_NT_PTR_ARRAY)
-		obj = alloc_parser_obj(args->parser);
-	else if (args->parser->model == PARSER_MODEL_NT_ARRAY)
-		obj = args->sarray + (args->parser->size * args->index);
+	if (array_parser->model == PARSER_MODEL_NT_PTR_ARRAY)
+		obj = alloc_parser_obj(parser);
+	else if (array_parser->model == PARSER_MODEL_NT_ARRAY)
+		obj = args->sarray + (parser->size * args->index);
 
-	if ((rc = parse(obj, NO_VAL, args->parser, src, args->args, ppath))) {
+	if ((rc = parse(obj, NO_VAL, parser, src, args->args, ppath))) {
 		log_flag(DATA, "%s object at 0x%"PRIxPTR" freed due to parser error: %s",
-			 args->parser->obj_type_string, (uintptr_t) obj,
+			 parser->obj_type_string, (uintptr_t) obj,
 			 slurm_strerror(rc));
-		free_parser_obj(args->parser, obj);
+		if (array_parser->model == PARSER_MODEL_NT_PTR_ARRAY)
+			free_parser_obj(parser, obj);
 		FREE_NULL_DATA(ppath);
 		return DATA_FOR_EACH_FAIL;
 	}
 
-	if (args->parser->model == PARSER_MODEL_NT_PTR_ARRAY) {
+	if (array_parser->model == PARSER_MODEL_NT_PTR_ARRAY) {
 		xassert(!args->array[args->index]);
 		args->array[args->index] = obj;
 	}
@@ -496,7 +510,7 @@ static int _parse_nt_array(const parser_t *const parser, void *dst, data_t *src,
 	foreach_nt_array_t fargs = {
 		.magic = MAGIC_FOREACH_NT_ARRAY,
 		.array_parser = parser,
-		.parser = find_parser_by_type(parser->pointer_type),
+		.parser = find_parser_by_type(parser->array_type),
 		.args = args,
 		.parent_path = parent_path,
 		.index = -1,
@@ -521,7 +535,11 @@ static int _parse_nt_array(const parser_t *const parser, void *dst, data_t *src,
 				      sizeof(*fargs.array));
 	else if (parser->model == PARSER_MODEL_NT_ARRAY)
 		fargs.sarray = xcalloc(data_get_list_length(src) + 1,
-				       sizeof(fargs.parser->size));
+				       fargs.parser->size);
+
+	/* verify new array actually allocated */;
+	xassert((fargs.array && (xsize(fargs.array) > 0)) ^
+		(fargs.sarray && (xsize(fargs.sarray) > 0)));
 
 	if (data_get_type(src) == DATA_TYPE_LIST) {
 		if (data_list_for_each(src, _foreach_array_entry, &fargs) < 0)
@@ -550,6 +568,8 @@ cleanup:
 		for (int i = 0; fargs.array[i]; i++)
 			free_parser_obj(parser, &fargs.array[i]);
 		xfree(fargs.array);
+	} else if (fargs.sarray) {
+		xfree(fargs.sarray);
 	}
 
 	return rc;
@@ -592,6 +612,10 @@ static void _parser_linked_flag(args_t *args, const parser_t *const array,
 			_set_flag_bit_equal(parser, dst, bit, matched, path,
 					    src);
 		*set |= bit->mask;
+	} else if (bit->type == FLAG_BIT_TYPE_REMOVED) {
+		if (matched && !is_fast_mode(args))
+			on_warn(PARSING, parser->type, args, path, __func__,
+				"Ignoring deprecated flag: %s", bit->name);
 	} else {
 		fatal_abort("%s: invalid bit_flag_t", __func__);
 	}
@@ -891,12 +915,13 @@ static void _parse_check_openapi(const parser_t *const parser, data_t *src,
 	 */
 	on_warn(PARSING, parser->type, args,
 		set_source_path(&path, args, parent_path), __func__,
-		"Expected OpenAPI type=%s%s%s (Slurm type=%s) but got OpenAPI type=%s%s%s (Slurm type=%s)",
+		"Expected OpenAPI type=%s%s%s (Slurm type=%s) but got OpenAPI type=%s%s%s (Slurm type=%s): %pd",
 		oas_type, (oas_format ? " format=" : ""),
 		(oas_format ? oas_format : ""),
 		data_type_to_string(oas_data_type), found_type,
 		(found_format ? " format=" : ""),
-		(found_format ? found_format : ""), data_get_type_string(src));
+		(found_format ? found_format : ""), data_get_type_string(src),
+		src);
 
 	xfree(path);
 }
@@ -916,7 +941,8 @@ extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 	 * Make sure the target object is the same size since there is no
 	 * way to dump value of __typeof__ as a value in C99.
 	 */
-	xassert((dst_bytes == NO_VAL) || (dst_bytes == parser->size));
+	xassert((dst_bytes == NO_VAL) || (dst_bytes == parser->size) ||
+		(parser->model == PARSER_MODEL_ALIAS));
 
 	if ((rc = load_prereqs(PARSING, parser, args)))
 		goto cleanup;
@@ -961,6 +987,12 @@ extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 		 parser->type_string, (uintptr_t) parser);
 
 	switch (parser->model) {
+	case PARSER_MODEL_REMOVED:
+		if (data_get_type(src) != DATA_TYPE_NULL)
+			on_warn(PARSING, parser->type, args, path, __func__,
+				"Ignoring value for removed parser");
+		rc = SLURM_SUCCESS;
+		break;
 	case PARSER_MODEL_FLAG_ARRAY:
 		verify_parser_not_sliced(parser);
 		rc = _parse_flag(dst, parser, src, args, parent_path);
@@ -969,7 +1001,7 @@ extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 		xassert(parser->list_type > DATA_PARSER_TYPE_INVALID);
 		xassert(parser->list_type < DATA_PARSER_TYPE_MAX);
 		verify_parser_not_sliced(parser);
-		xassert((dst_bytes == NO_VAL) || (dst_bytes == sizeof(List)));
+		xassert((dst_bytes == NO_VAL) || (dst_bytes == sizeof(list_t *)));
 		xassert(!parser->parse);
 		rc = _parse_list(parser, dst, src, args, parent_path);
 		break;
@@ -1028,6 +1060,11 @@ extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 
 		rc = parser->parse(parser, dst, src, args, parent_path);
 		break;
+	case PARSER_MODEL_ALIAS:
+		rc = parse(dst, dst_bytes,
+			   find_parser_by_type(parser->alias_type), src, args,
+			   parent_path);
+		break;
 	case PARSER_MODEL_ARRAY_LINKED_EXPLODED_FLAG_ARRAY_FIELD:
 	case PARSER_MODEL_ARRAY_LINKED_FIELD:
 		fatal_abort("%s: link model not allowed %u",
@@ -1124,10 +1161,15 @@ static void _dump_flag_bit_array_flag(args_t *args, void *src, data_t *dst,
 {
 	bool found;
 
+	if (bit->hidden)
+		return;
+
 	if (bit->type == FLAG_BIT_TYPE_BIT)
 		found = _match_flag_bit(parser, src, bit, *used_equal_bits);
 	else if (bit->type == FLAG_BIT_TYPE_EQUAL)
 		found = _match_flag_equal(parser, src, bit, used_equal_bits);
+	else if (bit->type == FLAG_BIT_TYPE_REMOVED)
+		found = false;
 	else
 		fatal_abort("%s: invalid bit_flag_t", __func__);
 
@@ -1169,6 +1211,8 @@ static void _dump_flag_bit_array_flag(args_t *args, void *src, data_t *dst,
 			type = "bit";
 		else if (bit->type == FLAG_BIT_TYPE_EQUAL)
 			type = "bit-equals";
+		else if (bit->type == FLAG_BIT_TYPE_REMOVED)
+			type = "removed";
 		else
 			type = "INVALID";
 
@@ -1220,8 +1264,9 @@ static int _foreach_dump_list(void *obj, void *arg)
 	xassert(args->parser->ptr_offset == NO_VAL);
 
 	/* we don't know the size of the items in the list */
-	if (dump(&obj, NO_VAL, find_parser_by_type(args->parser->list_type),
-		 item, args->args))
+	if (dump(&obj, NO_VAL, NULL,
+		 find_parser_by_type(args->parser->list_type), item,
+		 args->args))
 		return -1;
 
 	return 0;
@@ -1230,7 +1275,7 @@ static int _foreach_dump_list(void *obj, void *arg)
 static int _dump_list(const parser_t *const parser, void *src, data_t *dst,
 		      args_t *args)
 {
-	List *list_ptr = src;
+	list_t **list_ptr = src;
 	foreach_list_t fargs = {
 		.magic = MAGIC_FOREACH_LIST,
 		.args = args,
@@ -1262,21 +1307,27 @@ static int _dump_list(const parser_t *const parser, void *src, data_t *dst,
 	return SLURM_SUCCESS;
 }
 
-static int _dump_pointer(const parser_t *const parser, void *src, data_t *dst,
+static int _dump_pointer(const parser_t *const field_parser,
+			 const parser_t *const parser, void *src, data_t *dst,
 			 args_t *args)
 {
 	const parser_t *pt = find_parser_by_type(parser->pointer_type);
 	void **ptr = src;
 
-	if (!*ptr && !is_complex_mode(args)) {
-		/* Fully resolve pointer on NULL to use correct model */
-		while (pt->pointer_type)
-			pt = find_parser_by_type(pt->pointer_type);
+	if (!*ptr) {
+		if (is_complex_mode(args)) {
+			xassert(data_get_type(dst) == DATA_TYPE_NULL);
+			return SLURM_SUCCESS;
+		}
 
-		if (parser->allow_null_pointer) {
+		/* Fully resolve pointer on NULL to use correct model */
+		pt = unalias_parser(pt);
+
+		if (parser->allow_null_pointer ||
+		    (field_parser && !field_parser->required)) {
 			xassert(data_get_type(dst) == DATA_TYPE_NULL);
 		} else if ((pt->model == PARSER_MODEL_ARRAY) ||
-		    (pt->obj_openapi == OPENAPI_FORMAT_OBJECT)) {
+			   (pt->obj_openapi == OPENAPI_FORMAT_OBJECT)) {
 			/*
 			 * OpenAPI clients can't handle a null instead of an
 			 * object. Work around by placing an empty dictionary
@@ -1298,7 +1349,7 @@ static int _dump_pointer(const parser_t *const parser, void *src, data_t *dst,
 		return SLURM_SUCCESS;
 	}
 
-	return dump(*ptr, NO_VAL, pt, dst, args);
+	return dump(*ptr, NO_VAL, NULL, pt, dst, args);
 }
 
 static int _dump_nt_array(const parser_t *const parser, void *src, data_t *dst,
@@ -1316,7 +1367,7 @@ static int _dump_nt_array(const parser_t *const parser, void *src, data_t *dst,
 			return SLURM_SUCCESS;
 
 		for (int i = 0; !rc && array[i]; i++) {
-			rc = dump(array[i], NO_VAL,
+			rc = dump(array[i], NO_VAL, NULL,
 				  find_parser_by_type(parser->array_type),
 				  data_list_append(dst), args);
 		}
@@ -1340,7 +1391,7 @@ static int _dump_nt_array(const parser_t *const parser, void *src, data_t *dst,
 			if (done)
 				break;
 
-			rc = dump(ptr, NO_VAL,
+			rc = dump(ptr, NO_VAL, NULL,
 				  find_parser_by_type(parser->array_type),
 				  data_list_append(dst), args);
 		}
@@ -1351,6 +1402,58 @@ static int _dump_nt_array(const parser_t *const parser, void *src, data_t *dst,
 	return rc;
 }
 
+static void _dump_removed(const parser_t *parser, data_t *dst, args_t *args)
+{
+	if (is_complex_mode(args)) {
+		data_set_null(dst);
+		return;
+	}
+
+	while ((parser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD) ||
+	       parser->pointer_type) {
+		parser = unalias_parser(parser);
+
+		while (parser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD)
+			parser = find_parser_by_type(parser->type);
+	}
+
+	xassert(parser->model != PARSER_MODEL_ARRAY_REMOVED_FIELD);
+	xassert(parser->model !=
+		PARSER_MODEL_ARRAY_LINKED_EXPLODED_FLAG_ARRAY_FIELD);
+	xassert(parser->model > PARSER_MODEL_INVALID);
+	xassert(parser->model < PARSER_MODEL_MAX);
+
+	switch (parser->obj_openapi) {
+	case OPENAPI_FORMAT_INT:
+	case OPENAPI_FORMAT_INT32:
+	case OPENAPI_FORMAT_INT64:
+		data_set_int(dst, 0);
+		break;
+	case OPENAPI_FORMAT_NUMBER:
+	case OPENAPI_FORMAT_FLOAT:
+	case OPENAPI_FORMAT_DOUBLE:
+		data_set_float(dst, 0);
+		break;
+	case OPENAPI_FORMAT_STRING:
+	case OPENAPI_FORMAT_PASSWORD:
+		data_set_string(dst, "");
+		break;
+	case OPENAPI_FORMAT_BOOL:
+		data_set_bool(dst, false);
+	case OPENAPI_FORMAT_OBJECT:
+		data_set_dict(dst);
+		break;
+	case OPENAPI_FORMAT_ARRAY:
+		data_set_list(dst);
+		break;
+	case OPENAPI_FORMAT_MAX:
+	case OPENAPI_FORMAT_INVALID:
+		/* Should never happen but avoid crashing clients */
+		xassert(false);
+		data_set_null(dst);
+	};
+}
+
 static int _dump_linked(args_t *args, const parser_t *const array,
 			const parser_t *const parser, void *src, data_t *dst)
 {
@@ -1384,22 +1487,12 @@ static int _dump_linked(args_t *args, const parser_t *const array,
 			 (uintptr_t) src, array->field_name,
 			 array->ptr_offset, (uintptr_t) dst,
 			 array->key, (uintptr_t) dst);
-		goto cleanup;
+		return SLURM_SUCCESS;
 	}
 
 	if (parser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD) {
-		const parser_t *rparser = find_parser_by_type(parser->type);
-
-		while ((rparser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD) ||
-		       rparser->pointer_type) {
-			while (rparser->pointer_type)
-				rparser = find_parser_by_type(
-					rparser->pointer_type);
-
-			while (rparser->model ==
-			       PARSER_MODEL_ARRAY_REMOVED_FIELD)
-				rparser = find_parser_by_type(rparser->type);
-		}
+		const parser_t *const rparser =
+			find_parser_by_type(parser->type);
 
 		log_flag(DATA, "removed: %s parser %s->%s(0x%" PRIxPTR ") for %s(0x%" PRIxPTR ") for data(0x%" PRIxPTR ")/%s(0x%" PRIxPTR ")",
 			 parser->obj_type_string,
@@ -1409,33 +1502,7 @@ static int _dump_linked(args_t *args, const parser_t *const array,
 			 (uintptr_t) src, (uintptr_t) dst,
 			 array->key, (uintptr_t) dst);
 
-		switch (rparser->obj_openapi) {
-			case OPENAPI_FORMAT_INT:
-			case OPENAPI_FORMAT_INT32:
-			case OPENAPI_FORMAT_INT64:
-				data_set_int(dst, 0);
-				break;
-			case OPENAPI_FORMAT_NUMBER:
-			case OPENAPI_FORMAT_FLOAT:
-			case OPENAPI_FORMAT_DOUBLE:
-				data_set_float(dst, 0);
-				break;
-			case OPENAPI_FORMAT_STRING:
-			case OPENAPI_FORMAT_PASSWORD:
-				data_set_string(dst, "");
-				break;
-			case OPENAPI_FORMAT_BOOL:
-				data_set_bool(dst, false);
-			case OPENAPI_FORMAT_OBJECT:
-				data_set_dict(dst);
-				break;
-			case OPENAPI_FORMAT_ARRAY:
-				data_set_list(dst);
-				break;
-			case OPENAPI_FORMAT_MAX:
-			case OPENAPI_FORMAT_INVALID:
-				fatal_abort("invalid type");
-		};
+		_dump_removed(rparser, dst, args);
 
 		return SLURM_SUCCESS;
 	}
@@ -1449,6 +1516,10 @@ static int _dump_linked(args_t *args, const parser_t *const array,
 
 		for (int i = 0; i < parser->flag_bit_array_count; i++) {
 			const flag_bit_t *bit = &parser->flag_bit_array[i];
+
+			if (bit->hidden)
+				continue;
+
 			data_t *bit_dst = data_define_dict_path(dst, bit->name);
 
 			xassert(src);
@@ -1456,7 +1527,7 @@ static int _dump_linked(args_t *args, const parser_t *const array,
 						  bit, true, &used_equal_bits);
 		}
 
-		goto cleanup;
+		return SLURM_SUCCESS;
 	}
 
 	xassert(parser->model == PARSER_MODEL_ARRAY_LINKED_FIELD);
@@ -1468,7 +1539,8 @@ static int _dump_linked(args_t *args, const parser_t *const array,
 		 array->ptr_offset, (uintptr_t) dst, array->key,
 		 (uintptr_t) dst);
 
-	rc = dump(src, NO_VAL, find_parser_by_type(parser->type), dst, args);
+	rc = dump(src, NO_VAL, parser, find_parser_by_type(parser->type), dst,
+		  args);
 
 	log_flag(DATA, "END: dumping %s parser %s->%s(0x%" PRIxPTR ") for %s(0x%" PRIxPTR ")->%s(+%zd) for data(0x%" PRIxPTR ")/%s(0x%" PRIxPTR ")",
 		 parser->obj_type_string, array->type_string,
@@ -1477,7 +1549,6 @@ static int _dump_linked(args_t *args, const parser_t *const array,
 		 array->ptr_offset, (uintptr_t) dst, array->key,
 		 (uintptr_t) dst);
 
-cleanup:
 	return rc;
 }
 
@@ -1496,8 +1567,9 @@ static void _check_dump(const parser_t *const parser, data_t *dst, args_t *args)
 	}
 }
 
-extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
-		data_t *dst, args_t *args)
+extern int dump(void *src, ssize_t src_bytes,
+		const parser_t *const field_parser,
+		const parser_t *const parser, data_t *dst, args_t *args)
 {
 	int rc;
 
@@ -1516,7 +1588,8 @@ extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
 	 * Make sure the source object is the same size since there is no
 	 * way to dump value of __typeof__ as a value in C.
 	 */
-	xassert((src_bytes == NO_VAL) || (src_bytes == parser->size));
+	xassert((src_bytes == NO_VAL) || (src_bytes == parser->size) ||
+		(parser->model == PARSER_MODEL_ALIAS));
 
 	if (args->flags & FLAG_SPEC_ONLY) {
 		set_openapi_schema(dst, parser, args);
@@ -1527,6 +1600,10 @@ extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
 		goto done;
 
 	switch (parser->model) {
+	case PARSER_MODEL_REMOVED:
+		_dump_removed(parser, dst, args);
+		rc = SLURM_SUCCESS;
+		break;
 	case PARSER_MODEL_FLAG_ARRAY:
 		verify_parser_not_sliced(parser);
 		xassert((data_get_type(dst) == DATA_TYPE_NULL) ||
@@ -1554,7 +1631,7 @@ extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
 		verify_parser_not_sliced(parser);
 		xassert((data_get_type(dst) == DATA_TYPE_NULL) ||
 			(data_get_type(dst) == DATA_TYPE_LIST));
-		xassert((src_bytes == NO_VAL) || (src_bytes == sizeof(List)));
+		xassert((src_bytes == NO_VAL) || (src_bytes == sizeof(list_t *)));
 		xassert(!parser->dump);
 		rc = _dump_list(parser, src, dst, args);
 		break;
@@ -1564,7 +1641,7 @@ extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
 		verify_parser_not_sliced(parser);
 		xassert(data_get_type(dst) == DATA_TYPE_NULL);
 
-		rc = _dump_pointer(parser, src, dst, args);
+		rc = _dump_pointer(field_parser, parser, src, dst, args);
 		break;
 	case PARSER_MODEL_NT_PTR_ARRAY:
 	case PARSER_MODEL_NT_ARRAY:
@@ -1590,6 +1667,10 @@ extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
 		rc = parser->dump(parser, src, dst, args);
 		_check_dump(parser, dst, args);
 		break;
+	case PARSER_MODEL_ALIAS:
+		rc = dump(src, src_bytes, NULL,
+			  find_parser_by_type(parser->alias_type), dst, args);
+		break;
 	case PARSER_MODEL_ARRAY_LINKED_EXPLODED_FLAG_ARRAY_FIELD:
 	case PARSER_MODEL_ARRAY_LINKED_FIELD:
 		fatal_abort("%s: link model not allowed %u",
