@@ -44,7 +44,7 @@ extern int task_cgroup_cpuset_init(void);
 extern int task_cgroup_cpuset_fini(void);
 
 /* create user/job/jobstep cpuset cgroups */
-extern int task_cgroup_cpuset_create(stepd_step_rec_t *job);
+extern int task_cgroup_cpuset_create(stepd_step_rec_t *step);
 
 /* add a pid to the cgroup */
 extern int task_cgroup_cpuset_add_pid(pid_t pid);
