@@ -39,25 +39,38 @@ static uint16_t *avail_cores_per_sock = NULL;
 
 static uint64_t _shared_gres_task_limit(gres_job_state_t *gres_js,
 					bool use_total_gres,
+					bool one_task_sharing,
 					gres_node_state_t *gres_ns)
 {
-	int task_limit = 0, cnt;
+	int task_limit = 0, cnt, task_cnt;
+	gres_node_state_t *alt_gres_ns =
+		gres_ns->alt_gres ? gres_ns->alt_gres->gres_data : NULL;
+
 	for (int i = 0; i < gres_ns->topo_cnt; i++)
 	{
 		if (gres_js->type_id &&
 		    gres_js->type_id != gres_ns->topo_type_id[i])
 			continue;
-
+		if (!use_total_gres &&
+		    alt_gres_ns && alt_gres_ns->gres_bit_alloc &&
+		    gres_ns->topo_gres_bitmap && gres_ns->topo_gres_bitmap[i] &&
+		    bit_overlap_any(gres_ns->topo_gres_bitmap[i],
+				    alt_gres_ns->gres_bit_alloc))
+			continue; /* Skip alt gres that are currently used */
 		cnt = gres_ns->topo_gres_cnt_avail[i];
 
 		if (!use_total_gres)
 			cnt -= gres_ns->topo_gres_cnt_alloc[i];
 
-		if ((slurm_conf.select_type_param & MULTIPLE_SHARING_GRES_PJ))
-			task_limit += cnt / gres_js->gres_per_task;
+		if (one_task_sharing)
+			task_cnt = (cnt >= gres_js->gres_per_task) ? 1 : 0;
 		else
-			task_limit = MAX(task_limit,
-					 (cnt / gres_js->gres_per_task));
+			task_cnt = cnt / gres_js->gres_per_task;
+
+		if ((slurm_conf.select_type_param & MULTIPLE_SHARING_GRES_PJ))
+			task_limit += task_cnt;
+		else
+			task_limit = MAX(task_limit, task_cnt);
 	}
 	return task_limit;
 }
@@ -101,8 +114,8 @@ static void _estimate_cpus_per_gres(uint32_t ntasks_per_job,
 
 static int _sort_sockets_by_avail_cores(const void *x, const void *y)
 {
-	return slurm_sort_uint_list_desc(&avail_cores_per_sock[*(int *)x],
-					 &avail_cores_per_sock[*(int *)y]);
+	return slurm_sort_uint16_list_desc(&avail_cores_per_sock[*(int *)x],
+					   &avail_cores_per_sock[*(int *)y]);
 }
 
 static int _sock_gres_sort(void *x, void *y)
@@ -249,7 +262,7 @@ static void _reduce_restricted_cores(bitstr_t *avail_core,
 
 extern void gres_filter_sock_core(job_record_t *job_ptr,
 				  gres_mc_data_t *mc_ptr,
-				  List sock_gres_list,
+				  list_t *sock_gres_list,
 				  uint16_t sockets,
 				  uint16_t cores_per_socket,
 				  uint16_t cpus_per_core,
@@ -284,6 +297,9 @@ extern void gres_filter_sock_core(job_record_t *job_ptr,
 	if (*max_tasks_this_node == 0)
 		return;
 
+	if (mc_ptr->threads_per_core)
+		cpus_per_core = MIN(cpus_per_core, mc_ptr->threads_per_core);
+
 	xassert(avail_core);
 	avail_cores_per_sock = xcalloc(sockets, sizeof(uint16_t));
 	for (int s = 0; s < sockets; s++) {
@@ -310,7 +326,6 @@ extern void gres_filter_sock_core(job_record_t *job_ptr,
 		uint16_t res_core_tot = 0;
 		uint16_t cpus_per_gres = 0;
 		int min_core_cnt, req_cores, rem_sockets, req_sock_cnt = 0;
-		int threads_per_core;
 		bool is_res_gpu = false;
 
 		/*
@@ -326,12 +341,6 @@ extern void gres_filter_sock_core(job_record_t *job_ptr,
 			sock_gres->total_cnt =
 				sock_gres->total_cnt_before_filter;
 
-		if (mc_ptr->threads_per_core)
-			threads_per_core =
-				MIN(cpus_per_core,
-				    mc_ptr->threads_per_core);
-		else
-			threads_per_core = cpus_per_core;
 
 		if (!sock_gres->gres_state_job)
 			continue;
@@ -602,6 +611,8 @@ extern void gres_filter_sock_core(job_record_t *job_ptr,
 				    sock_gres->gres_state_job->config_flags))
 				max_tasks = _shared_gres_task_limit(
 					gres_js, sock_gres->use_total_gres,
+					(job_ptr->bit_flags &
+					 GRES_ONE_TASK_PER_SHARING),
 					sock_gres->gres_state_node->gres_data);
 			else
 				max_tasks = cnt_avail_total /
@@ -727,17 +738,17 @@ extern void gres_filter_sock_core(job_record_t *job_ptr,
 		efctv_cpt = mc_ptr->cpus_per_task;
 
 		if ((mc_ptr->ntasks_per_core == 1) &&
-		    (efctv_cpt % threads_per_core)) {
-			efctv_cpt /= threads_per_core;
+		    (efctv_cpt % cpus_per_core)) {
+			efctv_cpt /= cpus_per_core;
 			efctv_cpt++;
-			efctv_cpt *= threads_per_core;
+			efctv_cpt *= cpus_per_core;
 		}
 
 		req_cores *= efctv_cpt;
 
 		while (*max_tasks_this_node >= *min_tasks_this_node) {
 			/* round up by full threads per core */
-			req_cores = ROUNDUP(req_cores, threads_per_core);
+			req_cores = ROUNDUP(req_cores, cpus_per_core);
 			if (req_cores <= avail_cores_tot) {
 				if (removed_tasks)
 					log_flag(SELECT_TYPE, "Node %s: settings required_cores=%d by max_tasks_this_node=%u(reduced=%d) cpus_per_task=%d cpus_per_core=%d threads_per_core:%d",
@@ -863,11 +874,11 @@ extern void gres_filter_sock_core(job_record_t *job_ptr,
 		 * enforce_binding=false.
 		 */
 		if (enforce_binding &&
-		    ((req_cores * threads_per_core) > *avail_cpus)) {
-			log_flag(SELECT_TYPE, "Job cannot run on node %s: avail_cpus=%u < %u (required cores %u * threads_per_core %u",
+		    ((req_cores * cpus_per_core) > *avail_cpus)) {
+			log_flag(SELECT_TYPE, "Job cannot run on node %s: avail_cpus=%u < %u (required cores %u * cpus_per_core %u",
 				 node_name,
-				 *avail_cpus, req_cores * threads_per_core,
-				 req_cores, threads_per_core);
+				 *avail_cpus, req_cores * cpus_per_core,
+				 req_cores, cpus_per_core);
 			*max_tasks_this_node = 0;
 			break;
 		}
