@@ -55,20 +55,23 @@
 #include <grp.h>
 #include <pthread.h>
 #include <sched.h>
+#include <signal.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/mman.h>
 #include <sys/param.h>
 #include <sys/resource.h>
+#include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/utsname.h>
 #include <unistd.h>
 
+#include "slurm/slurm_errno.h"
+
 #include "src/common/assoc_mgr.h"
 #include "src/common/bitstring.h"
-#include "src/common/conmgr.h"
 #include "src/common/cpu_frequency.h"
 #include "src/common/daemonize.h"
 #include "src/common/fd.h"
@@ -89,16 +92,19 @@
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_pack.h"
 #include "src/common/slurm_rlimits_info.h"
+#include "src/common/slurm_time.h"
 #include "src/common/spank.h"
 #include "src/common/stepd_api.h"
 #include "src/common/uid.h"
 #include "src/common/xmalloc.h"
-#include "src/common/xsignal.h"
 #include "src/common/xstring.h"
 #include "src/common/xsystemd.h"
 
+#include "src/conmgr/conmgr.h"
+
 #include "src/interfaces/acct_gather_energy.h"
 #include "src/interfaces/auth.h"
+#include "src/interfaces/certmgr.h"
 #include "src/interfaces/cgroup.h"
 #include "src/interfaces/cred.h"
 #include "src/interfaces/gpu.h"
@@ -128,7 +134,15 @@
 
 decl_static_data(usage_txt);
 
+
+
 #define MAX_THREADS		256
+#define TIMEOUT_SIGUSR2 5000000
+#define TIMEOUT_RECONFIG 5000000
+#define SLURMD_CONMGR_DEFAULT_THREADS 10
+#define SLURMD_CONMGR_DEFAULT_MAX_CONNECTIONS 50
+#define MAX_THREAD_DELAY_INC ((timespec_t) { .tv_nsec = 1500, })
+#define MAX_THREAD_DELAY_MAX ((timespec_t) { .tv_sec = 1, })
 
 #define _free_and_set(__dst, __src)		\
 	do {					\
@@ -147,6 +161,14 @@ pthread_mutex_t tres_mutex     = PTHREAD_MUTEX_INITIALIZER;
 pthread_cond_t  tres_cond      = PTHREAD_COND_INITIALIZER;
 bool tres_packed = false;
 
+#define SERVICE_CONNECTION_ARGS_MAGIC 0x2aeaa8af
+typedef struct {
+	int magic; /* SERVICE_CONNECTION_ARGS_MAGIC */
+	timespec_t delay;
+	slurm_addr_t addr;
+	int fd;
+} service_connection_args_t;
+
 /*
  * count of active threads
  */
@@ -154,11 +176,6 @@ static int             active_threads = 0;
 static pthread_mutex_t active_mutex   = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t  active_cond    = PTHREAD_COND_INITIALIZER;
 
-typedef struct connection {
-	int fd;
-	slurm_addr_t *cli_addr;
-} conn_t;
-
 /*
  * Global data for resource specialization
  */
@@ -178,9 +195,6 @@ static int	ncpus;			/* number of CPUs on this node */
 static bool original = true;
 static bool under_systemd = false;
 static sig_atomic_t _shutdown = 0;
-static sig_atomic_t _reconfig = 0;
-static sig_atomic_t _update_log = 0;
-static pthread_t msg_pthread = (pthread_t) 0;
 static time_t sent_reg_time = (time_t) 0;
 
 /*
@@ -194,16 +208,14 @@ pthread_mutex_t cached_features_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static int       _convert_spec_cores(void);
 static int       _core_spec_init(void);
-static void      _create_msg_socket(void);
+static void _create_msg_socket(void);
 static void      _decrement_thd_count(void);
 static void      _destroy_conf(void);
 static void      _fill_registration_msg(slurm_node_registration_status_msg_t *);
-static void      _handle_connection(int fd, slurm_addr_t *client);
-static void      _hup_handler(int);
-static void      _increment_thd_count(void);
+static int _get_tls_certificate(void);
+static int _increment_thd_count(bool block);
 static void      _init_conf(void);
 static int       _memory_spec_init(void);
-static void      _msg_engine(void);
 static void _notify_parent_of_success(void);
 static void      _print_conf(void);
 static void      _print_config(void);
@@ -214,19 +226,18 @@ static void     *_registration_engine(void *arg);
 static void      _resource_spec_fini(void);
 static int       _resource_spec_init(void);
 static void      _select_spec_cores(void);
-static void     *_service_connection(void *);
 static int       _set_slurmd_spooldir(const char *dir);
 static int       _set_topo_info(void);
 static int       _set_work_dir(void);
 static int       _slurmd_init(void);
 static int       _slurmd_fini(void);
-static void _try_to_reconfig(void);
+static void *_try_to_reconfig(void *ptr);
 static void      _update_nice(void);
 static void      _usage(void);
-static void      _usr_handler(int);
 static int       _validate_and_convert_cpu_list(void);
 static void      _wait_for_all_threads(int secs);
 static void _wait_on_old_slurmd(bool kill_it);
+static void *_service_connection(void *arg);
 
 /**************************************************************************\
  * To test for memory leaks, set MEMORY_LEAK_DEBUG to 1 using
@@ -259,15 +270,79 @@ static void _wait_on_old_slurmd(bool kill_it);
  *    controller use).
 \**************************************************************************/
 
+static void _on_sigint(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGINT. Shutting down.");
+	slurmd_shutdown();
+}
+
+static void _on_sigterm(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGTERM. Shutting down.");
+	slurmd_shutdown();
+}
+
+static void _on_sigquit(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGQUIT. Shutting down.");
+	slurmd_shutdown();
+}
+
+static void _on_sigtstp(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGTSTP. Ignoring");
+}
+
+static void _on_sighup(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	if (conmgr_args.status == CONMGR_WORK_STATUS_CANCELLED)
+		return;
+
+	info("Caught SIGHUP. Triggering reconfigure.");
+
+	slurm_thread_create_detached(_try_to_reconfig, NULL);
+}
+
+static void _on_sigusr1(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGUSR1. Ignoring.");
+}
+
+static void _on_sigusr2(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	DEF_TIMERS;
+
+	if (conmgr_args.status == CONMGR_WORK_STATUS_CANCELLED)
+		return;
+
+	info("Caught SIGUSR2. Triggering logging update.");
+
+	START_TIMER;
+
+	update_slurmd_logging(LOG_LEVEL_END);
+	update_stepd_logging(false);
+
+	END_TIMER3(__func__, TIMEOUT_SIGUSR2);
+}
+
+static void _on_sigpipe(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGPIPE. Ignoring.");
+}
+
+static void _on_sigttin(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	debug("Caught SIGTTIN. Ignoring.");
+}
+
 int
 main (int argc, char **argv)
 {
-	int pidfd = -1;
-	int blocked_signals[] = {SIGPIPE, 0};
-	char *oom_value;
 	uint32_t curr_uid = 0;
-	char time_stamp[256];
 	log_options_t lopts = LOG_OPTS_INITIALIZER;
+	char *oom_value;
+	char time_stamp[256];
+	int pidfd;
 
 	if (getenv("SLURMD_RECONF"))
 		original = false;
@@ -298,6 +373,18 @@ main (int argc, char **argv)
 	conf->argv = argv;
 	conf->argc = argc;
 
+	/*
+	 * Process commandline arguments first, since one option may be
+	 * an alternate location for the slurm config file.
+	 */
+	_process_cmdline(conf->argc, conf->argv);
+
+	/*
+	 * Become a daemon if desired.
+	 */
+	if (original && conf->daemonize && xdaemon())
+		fatal_abort("%s: xdaemon() failed: %m", __func__);
+
 	if (_slurmd_init() < 0) {
 		error( "slurmd initialization failed" );
 		fflush( NULL );
@@ -315,25 +402,26 @@ main (int argc, char **argv)
 		      slurmd_user, slurm_conf.slurm_user_id, curr_user);
 	}
 
-	xsignal(SIGTERM, slurmd_shutdown);
-	xsignal(SIGINT,  slurmd_shutdown);
-	xsignal(SIGHUP,  _hup_handler);
-	xsignal(SIGUSR2, _usr_handler);
-	xsignal_block(blocked_signals);
-
 	debug3("slurmd initialization successful");
 
-	/*
-	 * Become a daemon if desired.
-	 */
-	if (original && conf->daemonize) {
-		if (xdaemon())
-			error("Couldn't daemonize slurmd: %m");
-	}
 	test_core_limit();
 	info("slurmd version %s started", SLURM_VERSION_STRING);
 	debug3("finished daemonize");
 
+	conmgr_init(SLURMD_CONMGR_DEFAULT_THREADS,
+		    SLURMD_CONMGR_DEFAULT_MAX_CONNECTIONS,
+		    (conmgr_callbacks_t) {0});
+
+	conmgr_add_work_signal(SIGINT, _on_sigint, NULL);
+	conmgr_add_work_signal(SIGTERM, _on_sigterm, NULL);
+	conmgr_add_work_signal(SIGQUIT, _on_sigquit, NULL);
+	conmgr_add_work_signal(SIGTSTP, _on_sigtstp, NULL);
+	conmgr_add_work_signal(SIGHUP, _on_sighup, NULL);
+	conmgr_add_work_signal(SIGUSR1, _on_sigusr1, NULL);
+	conmgr_add_work_signal(SIGUSR2, _on_sigusr2, NULL);
+	conmgr_add_work_signal(SIGPIPE, _on_sigpipe, NULL);
+	conmgr_add_work_signal(SIGTTIN, _on_sigttin, NULL);
+
 	if ((oom_value = getenv("SLURMD_OOM_ADJ"))) {
 		int i = atoi(oom_value);
 		debug("Setting slurmd oom_adj to %d", i);
@@ -373,7 +461,11 @@ main (int argc, char **argv)
 	if (select_g_init(1) != SLURM_SUCCESS)
 		fatal("Failed to initialize select plugins.");
 	file_bcast_init();
-	run_command_init();
+	if ((run_command_init(argc, argv, conf->binary) != SLURM_SUCCESS) &&
+	    conf->binary[0])
+		fatal("%s: Unable to reliably execute %s",
+		      __func__, conf->binary);
+
 	plugins_registered = true;
 
 	_create_msg_socket();
@@ -401,8 +493,7 @@ main (int argc, char **argv)
 	record_launched_jobs();
 	slurm_thread_create_detached(_registration_engine, NULL);
 
-	/* main processing loop. when this returns start shutting down */
-	_msg_engine();
+	conmgr_run(true);
 
 	/*
 	 * Unlink now while the slurm_conf.pidfile is still accessible,
@@ -434,12 +525,87 @@ main (int argc, char **argv)
 	 * Explicitly close the pidfile after all other shutdown has completed
 	 * which will release the flock.
 	 */
-	if (pidfd >= 0)			/* valid pidfd, non-error */
-		(void) close(pidfd);	/* Ignore errors */
+	fd_close(&pidfd);
 
 	info("Slurmd shutdown completing");
+
+	conmgr_fini();
 	log_fini();
-       	return 0;
+
+	return SLURM_SUCCESS;
+}
+
+static void _get_tls_cert_work(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	if (_get_tls_certificate()) {
+		error("%s: Unable to get TLS certificate", __func__);
+	}
+}
+
+static int _get_tls_certificate(void)
+{
+	slurm_msg_t req, resp;
+	tls_cert_request_msg_t cert_req = { 0 };
+	tls_cert_response_msg_t *cert_resp;
+
+	slurm_msg_t_init(&req);
+	slurm_msg_t_init(&resp);
+
+	if (!certmgr_enabled()) {
+		log_flag(TLS, "certmgr not enabled, skipping process to get signed TLS certificate from slurmctld (assume node already has signed TLS certificate)");
+		return SLURM_SUCCESS;
+	}
+
+	/* Periodically renew TLS certificate indefinitely */
+	conmgr_add_work_delayed_fifo(
+		_get_tls_cert_work, NULL,
+		certmgr_get_renewal_period_mins() * MINUTE_SECONDS, 0);
+
+	if (!(cert_req.token = certmgr_g_get_node_token(conf->node_name))) {
+		error("%s: Failed to get unique node token", __func__);
+		return SLURM_ERROR;
+	}
+
+	if (!(cert_req.csr = certmgr_g_generate_csr(conf->node_name))) {
+		error("%s: Failed to generate certificate signing request",
+		      __func__);
+		return SLURM_ERROR;
+	}
+
+	cert_req.node_name = xstrdup(conf->node_name);
+
+	req.msg_type = REQUEST_TLS_CERT;
+	req.data = &cert_req;
+
+	if (slurm_send_recv_controller_msg(&req, &resp, working_cluster_rec)
+	    < 0) {
+		error("Unable to get TLS certificate from slurmctld: %m");
+		return SLURM_ERROR;
+	}
+
+	switch (resp.msg_type) {
+	case RESPONSE_TLS_CERT:
+		break;
+	case RESPONSE_SLURM_RC:
+	{
+		uint32_t resp_rc =
+			((return_code_msg_t *) resp.data)->return_code;
+		error("%s: slurmctld response to TLS certificate request: %s",
+		      __func__, slurm_strerror(resp_rc));
+		return SLURM_ERROR;
+	}
+	default:
+		error("%s: slurmctld responded with unexpected msg type: %s",
+		      __func__, rpc_num2string(resp.msg_type));
+		return SLURM_ERROR;
+	}
+
+	cert_resp = resp.data;
+
+	log_flag(TLS, "Successfully got signed certificate from slurmctld: \n%s",
+		 cert_resp->signed_cert);
+
+	return SLURM_SUCCESS;
 }
 
 /*
@@ -452,11 +618,14 @@ _registration_engine(void *arg)
 {
 	static const uint32_t MAX_DELAY = 128;
 	uint32_t delay = 1;
-	_increment_thd_count();
+	(void) _increment_thd_count(true);
 
 	while (!_shutdown && !sent_reg_time) {
 		int rc;
 
+		if (_get_tls_certificate())
+			error("Unable to get TLS certificate");
+
 		if (!(rc = send_registration_msg(SLURM_SUCCESS)))
 			break;
 
@@ -477,53 +646,6 @@ _registration_engine(void *arg)
 	return NULL;
 }
 
-static void _msg_engine(void)
-{
-	slurm_addr_t *cli;
-	int sock;
-
-	msg_pthread = pthread_self();
-	slurmd_req(NULL);	/* initialize timer */
-	while (!_shutdown) {
-		if (_reconfig) {
-			int rpc_wait = MAX(5, slurm_conf.msg_timeout / 2);
-			DEF_TIMERS;
-			START_TIMER;
-			verbose("got reconfigure request");
-			/* Wait for RPCs to finish */
-			_wait_for_all_threads(rpc_wait);
-			if (_shutdown)
-				break;
-			_try_to_reconfig();
-			END_TIMER3("_reconfigure request - slurmd doesn't accept new connections during this time.",
-				   5000000);
-		}
-		if (_update_log) {
-			DEF_TIMERS;
-			START_TIMER;
-			update_slurmd_logging(LOG_LEVEL_END);
-			update_stepd_logging(false);
-			END_TIMER3("_update_log request - slurmd doesn't accept new connections during this time.",
-				   5000000);
-		}
-		cli = xmalloc(sizeof(*cli));
-		if ((sock = slurm_accept_msg_conn(conf->lfd, cli)) >= 0) {
-			_handle_connection(sock, cli);
-			continue;
-		}
-		/*
-		 *  Otherwise, accept() failed.
-		 */
-		xfree(cli);
-		if (errno == EINTR)
-			continue;
-		error("accept: %m");
-	}
-	verbose("got shutdown request");
-	close(conf->lfd);
-	conf->lfd = -1;
-}
-
 static void _decrement_thd_count(void)
 {
 	slurm_mutex_lock(&active_mutex);
@@ -533,7 +655,12 @@ static void _decrement_thd_count(void)
 	slurm_mutex_unlock(&active_mutex);
 }
 
-static void _increment_thd_count(void)
+/*
+ * Increment thread count
+ * IN block - true to block on too many active threads
+ * RET SLURM_SUCCESS or EWOULDBLOCK
+ */
+static int _increment_thd_count(bool block)
 {
 	bool logged = false;
 
@@ -544,10 +671,17 @@ static void _increment_thd_count(void)
 			     MAX_THREADS);
 			logged = true;
 		}
-		slurm_cond_wait(&active_cond, &active_mutex);
+
+		if (block)  {
+			slurm_cond_wait(&active_cond, &active_mutex);
+		} else {
+			slurm_mutex_unlock(&active_mutex);
+			return EWOULDBLOCK;
+		}
 	}
 	active_threads++;
 	slurm_mutex_unlock(&active_mutex);
+	return SLURM_SUCCESS;
 }
 
 /* secs IN - wait up to this number of seconds for all threads to complete */
@@ -583,29 +717,24 @@ _wait_for_all_threads(int secs)
 	verbose("all threads complete");
 }
 
-static void _handle_connection(int fd, slurm_addr_t *cli)
+static void *_service_connection(void *arg)
 {
-	conn_t *arg = xmalloc(sizeof(conn_t));
-
-	arg->fd       = fd;
-	arg->cli_addr = cli;
-
-	_increment_thd_count();
-	slurm_thread_create_detached(_service_connection, arg);
-}
-
-static void *
-_service_connection(void *arg)
-{
-	conn_t *con = (conn_t *) arg;
-	slurm_msg_t *msg = xmalloc(sizeof(slurm_msg_t));
+	service_connection_args_t *args = arg;
+	slurm_msg_t *msg = NULL;
+	slurm_addr_t *addr = &args->addr;
+	int fd = args->fd;
 	int rc = SLURM_SUCCESS;
 
-	debug3("in the service_connection");
+	xassert(args->magic == SERVICE_CONNECTION_ARGS_MAGIC);
+
+	debug3("%s: [%pA] processing new RPC connection", __func__, &addr);
+
+	msg = xmalloc_nz(sizeof(*msg));
 	slurm_msg_t_init(msg);
+
 	msg->flags |= SLURM_MSG_KEEP_BUFFER;
-	if ((rc = slurm_receive_msg_and_forward(con->fd, con->cli_addr, msg))
-	   != SLURM_SUCCESS) {
+
+	if ((rc = slurm_receive_msg_and_forward(fd, addr, msg))) {
 		error("service_connection: slurm_receive_msg: %m");
 		/*
 		 * if this fails we need to make sure the nodes we forward
@@ -623,23 +752,24 @@ _service_connection(void *arg)
 	debug2("Start processing RPC: %s", rpc_num2string(msg->msg_type));
 
 	if (slurm_conf.debug_flags & DEBUG_FLAG_AUDIT_RPCS) {
-		slurm_addr_t cli_addr;
-		(void) slurm_get_peer_addr(con->fd, &cli_addr);
 		log_flag(AUDIT_RPCS, "msg_type=%s uid=%u client=[%pA] protocol=%u",
 			 rpc_num2string(msg->msg_type), msg->auth_uid,
-			 &cli_addr, msg->protocol_version);
+			 &addr, msg->protocol_version);
 	}
 
 	slurmd_req(msg);
 
 cleanup:
 	if ((msg->conn_fd >= 0) && close(msg->conn_fd) < 0)
-		error ("close(%d): %m", con->fd);
+		error ("close(%d): %m", fd);
 
-	xfree(con->cli_addr);
-	xfree(con);
 	debug2("Finish processing RPC: %s", rpc_num2string(msg->msg_type));
+
 	slurm_free_msg(msg);
+
+	args->magic = ~SERVICE_CONNECTION_ARGS_MAGIC;
+	xfree(args);
+
 	_decrement_thd_count();
 	return NULL;
 }
@@ -678,10 +808,10 @@ static void _handle_node_reg_resp(slurm_msg_t *resp_msg)
 	case RESPONSE_SLURM_RC:
 		rc = ((return_code_msg_t *) resp_msg->data)->return_code;
 		if (rc)
-			slurm_seterrno(rc);
+			errno = rc;
 		break;
 	default:
-		slurm_seterrno(SLURM_UNEXPECTED_MSG_ERROR);
+		errno = SLURM_UNEXPECTED_MSG_ERROR;
 		break;
 	}
 
@@ -809,7 +939,6 @@ _fill_registration_msg(slurm_node_registration_status_msg_t *msg)
 	char *arch, *os;
 	struct utsname buf;
 	static bool first_msg = true;
-	static time_t slurmd_start_time = 0;
 	buf_t *gres_info;
 
 	if (!sent_successful_registration) {
@@ -836,10 +965,7 @@ _fill_registration_msg(slurm_node_registration_status_msg_t *msg)
 	msg->sockets	 = conf->sockets;
 	msg->cores	 = conf->cores;
 	msg->threads	 = conf->threads;
-	if (res_abs_cpus[0] == '\0')
-		msg->cpu_spec_list = NULL;
-	else
-		msg->cpu_spec_list = xstrdup(res_abs_cpus);
+	msg->cpu_spec_list = xstrdup(conf->cpu_spec_list);
 	msg->real_memory = conf->physical_memory_size;
 	msg->tmp_disk    = conf->tmp_disk_space;
 	msg->hash_val = slurm_conf.hash_val;
@@ -1330,16 +1456,34 @@ rwfail:
 	return;
 }
 
-static void _try_to_reconfig(void)
+static void *_try_to_reconfig(void *ptr)
 {
 	extern char **environ;
 	struct rlimit rlim;
 	char **child_env;
 	pid_t pid;
 	int to_parent[2] = {-1, -1};
+	int rpc_wait = MAX(5, slurm_conf.msg_timeout);
+	int close_skip[] = { -1, -1, -1, -1 }, skip_index = 0, auth_fd = -1;
+	DEF_TIMERS;
 
-	_reconfig = 0;
-	conmgr_quiesce(true);
+	if ((auth_fd = auth_g_get_reconfig_fd(AUTH_PLUGIN_SLURM)) >= 0)
+		close_skip[skip_index++] = auth_fd;
+
+	conmgr_quiesce(__func__);
+
+	START_TIMER;
+
+	if (slurm_conf.prolog_epilog_timeout != NO_VAL16)
+		rpc_wait = MAX(rpc_wait, slurm_conf.prolog_epilog_timeout);
+
+	/* Wait for RPCs to finish */
+	_wait_for_all_threads(rpc_wait);
+
+	if (_shutdown) {
+		conmgr_unquiesce(__func__);
+		return NULL;
+	}
 
 	save_cred_state();
 
@@ -1358,21 +1502,21 @@ static void _try_to_reconfig(void)
 	if (conf->lfd != -1) {
 		setenvf(&child_env, "SLURMD_RECONF_LISTEN_FD", "%d", conf->lfd);
 		fd_set_noclose_on_exec(conf->lfd);
+		close_skip[skip_index++] = conf->lfd;
+		debug3("%s: retaining listener socket fd:%d", __func__, conf->lfd);
 	}
 
 	if (!conf->daemonize && !under_systemd)
 		goto start_child;
 
-	if (pipe(to_parent) < 0) {
-		error("%s: pipe() failed: %m", __func__);
-		return;
-	}
+	if (pipe(to_parent))
+		fatal("%s: pipe() failed: %m", __func__);
 
 	setenvf(&child_env, "SLURMD_RECONF_PARENT_FD", "%d", to_parent[1]);
+	close_skip[skip_index++] = to_parent[1];
 
 	if ((pid = fork()) < 0) {
-		error("%s: fork() failed, cannot reconfigure.", __func__);
-		return;
+		fatal("%s: fork() failed, cannot reconfigure.", __func__);
 	} else if (pid > 0) {
 		pid_t grandchild_pid;
 		int rc;
@@ -1402,15 +1546,14 @@ rwfail:
 		env_array_free(child_env);
 		waitpid(pid, &rc, 0);
 		info("Resuming operation, reconfigure failed.");
-		conmgr_run(false);
-		return;
+
+		END_TIMER3(__func__, TIMEOUT_RECONFIG);
+		conmgr_unquiesce(__func__);
+		return NULL;
 	}
 
 start_child:
-	for (int fd = 3; fd < rlim.rlim_cur; fd++) {
-		if ((fd != to_parent[1]) && (fd != conf->lfd))
-			(void) close(fd);
-	}
+	closeall_except(3, close_skip);
 
 	/*
 	 * This second fork() ensures that the new grandchild's parent is init,
@@ -1585,11 +1728,27 @@ static void
 _print_config(void)
 {
 	int days, hours, mins, secs;
-	char name[128];
+	char name[128], *gres_str = NULL, *autodetect_str = NULL;
+	node_config_load_t node_conf = {
+		/* Set cpu_cnt later */
+		.in_slurmd = true,
+		.gres_name = "gpu",
+		.xcpuinfo_mac_to_abs = xcpuinfo_mac_to_abs
+	};
+
+	/* Since it is not running the daemon, silence the log output */
+	(conf->log_opts).logfile_level = LOG_LEVEL_QUIET;
+	(conf->log_opts).syslog_level = LOG_LEVEL_QUIET;
+
+	/* Print to fatals to terminal by default (-v for more verbosity) */
+	if (conf->debug_level_set)
+		(conf->log_opts).stderr_level = conf->debug_level;
+	else
+		(conf->log_opts).stderr_level = LOG_LEVEL_FATAL;
+
+	log_alter(conf->log_opts, SYSLOG_FACILITY_USER, NULL);
 
 	gethostname_short(name, sizeof(name));
-	printf("NodeName=%s ", name);
-
 	xcpuinfo_hwloc_topo_get(&conf->actual_cpus,
 				&conf->actual_boards,
 				&conf->actual_sockets,
@@ -1597,14 +1756,27 @@ _print_config(void)
 				&conf->actual_threads,
 				&conf->block_map_size,
 				&conf->block_map, &conf->block_map_inv);
-	printf("CPUs=%u Boards=%u SocketsPerBoard=%u CoresPerSocket=%u "
-	       "ThreadsPerCore=%u ",
-	       conf->actual_cpus, conf->actual_boards,
-	       (conf->actual_sockets / conf->actual_boards),
-	       conf->actual_cores, conf->actual_threads);
+
+	/* Set sockets and cores for xcpuinfo_mac_to_abs */
+	conf->cpus = conf->actual_cpus;
+	conf->boards = conf->actual_boards;
+	conf->sockets = conf->actual_sockets;
+	conf->cores = conf->actual_cores;
+	conf->threads = conf->actual_threads;
+	node_conf.cpu_cnt = MAX(conf->actual_cpus, conf->block_map_size);
+	/* Use default_plugin_path here to avoid reading slurm.conf */
+	slurm_conf.plugindir = xstrdup(default_plugin_path);
+	gres_get_autodetected_gpus(node_conf, &gres_str, &autodetect_str);
 
 	get_memory(&conf->physical_memory_size);
-	printf("RealMemory=%"PRIu64"\n", conf->physical_memory_size);
+
+	printf("NodeName=%s CPUs=%u Boards=%u SocketsPerBoard=%u CoresPerSocket=%u ThreadsPerCore=%u RealMemory=%"PRIu64"%s%s\n",
+	       name, conf->actual_cpus, conf->actual_boards,
+	       (conf->actual_sockets / conf->actual_boards), conf->actual_cores,
+	       conf->actual_threads, conf->physical_memory_size,
+	       gres_str ? " Gres=" : "", gres_str ? gres_str : "");
+	if (autodetect_str)
+		printf("%s\n", autodetect_str);
 
 	get_up_time(&conf->up_time);
 	secs  =  conf->up_time % 60;
@@ -1621,7 +1793,9 @@ static void _print_gres(void)
 	o->logfile_level = LOG_LEVEL_QUIET;
 	o->stderr_level = LOG_LEVEL_INFO;
 	o->syslog_level = LOG_LEVEL_INFO;
-	o->prefix_level = false;
+	if (conf->debug_level_set)
+		o->stderr_level = conf->debug_level;
+
 	log_alter(conf->log_opts, SYSLOG_FACILITY_USER, NULL);
 
 	_load_gres();
@@ -1659,6 +1833,10 @@ _process_cmdline(int ac, char **av)
 		{NULL,			0,                 0, 0}
 	};
 
+	if (run_command_is_launcher(ac, av)) {
+		run_command_launcher(ac, av);
+		_exit(127); /* Should not get here */
+	}
 	conf->prog = xbasename(av[0]);
 
 	while ((c = getopt_long(ac, av, opt_string, long_options, NULL)) > 0) {
@@ -1814,18 +1992,160 @@ _process_cmdline(int ac, char **av)
 	}
 }
 
-static void _create_msg_socket(void)
+static void *_on_listen_connect(conmgr_fd_t *con, void *arg)
 {
-	if (getenv("SLURMD_RECONF_LISTEN_FD")) {
-		conf->lfd = atoi(getenv("SLURMD_RECONF_LISTEN_FD"));
-		debug2("%s: inherited socket on fd=%d", __func__, conf->lfd);
+	debug3("%s: [%s] Successfully opened slurm listen port %u",
+	       __func__, conmgr_fd_get_name(con), conf->port);
+
+	slurmd_req(NULL);	/* initialize timer */
+
+	return con;
+}
+
+static void _on_listen_finish(conmgr_fd_t *con, void *arg)
+{
+	xassert(con == arg);
+
+	debug3("%s: [%s] closed RPC listener. Queuing up cleanup.",
+	       __func__, conmgr_fd_get_name(con));
+
+	conf->lfd = -1;
+}
+
+/* Try to process connection if thread max has not been hit */
+static void _try_service_connection(conmgr_callback_args_t conmgr_args,
+				    void *arg)
+{
+	service_connection_args_t *args = arg;
+	int rc = SLURM_ERROR;
+
+	xassert(args->magic == SERVICE_CONNECTION_ARGS_MAGIC);
+
+	if (!(rc = _increment_thd_count(false))) {
+		debug3("%s: [%pA] detaching new thread for RPC connection",
+		       __func__, &args->addr);
+
+		slurm_thread_create_detached(_service_connection, args);
+	} else {
+		xassert(rc == EWOULDBLOCK);
+
+		debug3("%s: [%pA] deferring servicing connection",
+		       __func__, &args->addr);
+
+		/*
+		 * Backoff attempts to avoid needless lock contention while
+		 * avoiding having a new thread created
+		 */
+		args->delay = timespec_add(args->delay, MAX_THREAD_DELAY_INC);
+		if (timespec_is_after(args->delay, MAX_THREAD_DELAY_MAX))
+			args->delay = MAX_THREAD_DELAY_MAX;
+
+		conmgr_add_work_delayed_fifo(_try_service_connection, args,
+					     args->delay.tv_sec,
+					     args->delay.tv_sec);
+	}
+}
+
+static void _on_extract_fd(conmgr_callback_args_t conmgr_args,
+			   int input_fd, int output_fd, void *arg)
+{
+	service_connection_args_t *args = NULL;
+	int rc = SLURM_SUCCESS;
+
+	xassert(!arg);
+
+	if (conmgr_args.status == CONMGR_WORK_STATUS_CANCELLED) {
+		debug3("%s: [fd:%d] connection work cancelled",
+		       __func__, input_fd);
+
+		if (input_fd != output_fd)
+			fd_close(&output_fd);
+		fd_close(&input_fd);
 		return;
 	}
 
-	if ((conf->lfd = slurm_init_msg_engine_port(conf->port)) < 0)
-		fatal("Unable to bind listen port (%u): %m", conf->port);
+	if ((input_fd < 0) || (output_fd < 0)) {
+		error("%s: Rejecting partially open connection input_fd=%d output_fd=%d",
+		      __func__, input_fd, output_fd);
+		if (input_fd != output_fd)
+			fd_close(&output_fd);
+		fd_close(&input_fd);
+		return;
+	}
 
-	debug3("Successfully opened slurm listen port %u", conf->port);
+	args = xmalloc(sizeof(*args));
+	args->magic = SERVICE_CONNECTION_ARGS_MAGIC;
+	args->addr.ss_family = AF_UNSPEC;
+	args->fd = input_fd;
+
+	if ((rc = slurm_get_peer_addr(input_fd, &args->addr))) {
+		error("%s: [fd:%d] getting socket peer failed: %s",
+		      __func__, input_fd, slurm_strerror(rc));
+		fd_close(&input_fd);
+		args->magic = ~SERVICE_CONNECTION_ARGS_MAGIC;
+		xfree(args);
+		return;
+	}
+
+	/* force blocking mode for blocking handlers */
+	fd_set_blocking(input_fd);
+
+	_try_service_connection(conmgr_args, args);
+}
+
+static void *_on_connection(conmgr_fd_t *con, void *arg)
+{
+	int rc;
+
+	debug3("%s: [%s] New RPC connection",
+	       __func__, conmgr_fd_get_name(con));
+
+	if ((rc = conmgr_queue_extract_con_fd(con, _on_extract_fd,
+					      XSTRINGIFY(_on_extract_fd),
+					      NULL))) {
+		error("%s: [%s] Extracting FDs failed: %s",
+		      __func__, conmgr_fd_get_name(con), slurm_strerror(rc));
+		return NULL;
+	}
+
+	return con;
+}
+
+static void _on_finish(conmgr_fd_t *con, void *arg)
+{
+	xassert(arg == con);
+
+	debug3("%s: [%s] RPC connection closed",
+	       __func__, conmgr_fd_get_name(con));
+}
+
+static int _on_msg(conmgr_fd_t *con, slurm_msg_t *msg, int unpack_rc, void *arg)
+{
+	fatal_abort("should never happen");
+}
+
+static void _create_msg_socket(void)
+{
+	static const conmgr_events_t events = {
+		.on_listen_connect = _on_listen_connect,
+		.on_listen_finish = _on_listen_finish,
+		.on_connection = _on_connection,
+		.on_msg = _on_msg,
+		.on_finish = _on_finish,
+	};
+	int rc;
+
+	if (getenv("SLURMD_RECONF_LISTEN_FD")) {
+		conf->lfd = atoi(getenv("SLURMD_RECONF_LISTEN_FD"));
+		debug2("%s: inherited socket on fd:%d", __func__, conf->lfd);
+	} else if ((conf->lfd = slurm_init_msg_engine_port(conf->port)) < 0) {
+		fatal("Unable to bind listen port (%u): %m", conf->port);
+	}
+
+	if ((rc = conmgr_process_fd_listen(conf->lfd, CON_TYPE_RPC, &events,
+					   CON_FLAG_NONE, NULL)))
+		fatal("%s: unable to process fd:%d error:%s",
+		      __func__, conf->lfd, slurm_strerror(rc));
 }
 
 static void
@@ -2186,12 +2506,6 @@ _slurmd_init(void)
 	struct stat stat_buf;
 	int rc = SLURM_SUCCESS;
 
-	/*
-	 * Process commandline arguments first, since one option may be
-	 * an alternate location for the slurm config file.
-	 */
-	_process_cmdline(conf->argc, conf->argv);
-
 	/*
 	 * Work out how this node is going to be configured. If running in
 	 * "configless" mode, also populate the conf-cache directory.
@@ -2212,7 +2526,8 @@ _slurmd_init(void)
 		slurm_conf.debug_flags = DEBUG_FLAG_GRES;
 	if (gres_init() != SLURM_SUCCESS)
 		return SLURM_ERROR;
-	build_all_nodeline_info(true, 0);
+	if (build_all_nodeline_info(true, 0))
+		return SLURM_ERROR;
 	build_all_frontend_info(true);
 
 	/*
@@ -2222,8 +2537,22 @@ _slurmd_init(void)
 	if (cgroup_conf_init() != SLURM_SUCCESS)
 		log_flag(CGROUP, "cgroup conf was already initialized.");
 
+	/*
+	 * If we are in the process of daemonizing ourselves, do not refresh
+	 * the hwloc as the grandparent process have already done it. This
+	 * is important as we're already constrained by cgroups in a specific
+	 * cpuset, and hwloc does not return the correct e-cores vs p-cores
+	 * kinds.
+	 */
 	xcpuinfo_refresh_hwloc(original);
 
+	/*
+	 * auth/slurm calls conmgr_init and we need to apply conmgr params
+	 * before conmgr init.
+	 */
+	if (slurm_conf.slurmd_params)
+		conmgr_set_params(slurm_conf.slurmd_params);
+
 	/*
 	 * auth and hash plugins must be initialized before the first dynamic
 	 * future registration is send.
@@ -2232,6 +2561,8 @@ _slurmd_init(void)
 		return SLURM_ERROR;
 	if (hash_g_init() != SLURM_SUCCESS)
 		return SLURM_ERROR;
+	if (certmgr_g_init() != SLURM_SUCCESS)
+		return SLURM_ERROR;
 
 	_dynamic_init();
 
@@ -2313,7 +2644,7 @@ _slurmd_init(void)
 
 	if (proctrack_g_init() != SLURM_SUCCESS)
 		return SLURM_ERROR;
-	if (slurmd_task_init() != SLURM_SUCCESS)
+	if (task_g_init() != SLURM_SUCCESS)
 		return SLURM_ERROR;
 	if (spank_slurmd_init() < 0)
 		return SLURM_ERROR;
@@ -2377,11 +2708,12 @@ _slurmd_fini(void)
 	acct_gather_profile_fini();
 	cred_state_fini();
 	switch_g_fini();
-	slurmd_task_fini();
+	task_g_fini();
 	slurm_conf_destroy();
 	proctrack_g_fini();
 	auth_g_fini();
 	hash_g_fini();
+	certmgr_g_fini();
 	node_fini2();
 	gres_fini();
 	prep_g_fini();
@@ -2412,29 +2744,10 @@ _slurmd_fini(void)
 	return SLURM_SUCCESS;
 }
 
-extern void slurmd_shutdown(int signum)
+extern void slurmd_shutdown(void)
 {
-	if (signum == SIGTERM || signum == SIGINT) {
-		_shutdown = 1;
-		if (msg_pthread && (pthread_self() != msg_pthread))
-			pthread_kill(msg_pthread, SIGTERM);
-	}
-}
-
-static void
-_hup_handler(int signum)
-{
-	if (signum == SIGHUP) {
-		_reconfig = 1;
-	}
-}
-
-static void
-_usr_handler(int signum)
-{
-	if (signum == SIGUSR2) {
-		_update_log = 1;
-	}
+	_shutdown = 1;
+	conmgr_request_shutdown();
 }
 
 static void _usage(void)
@@ -2497,7 +2810,6 @@ extern void update_slurmd_logging(log_level_t log_lvl)
 	log_options_t *o = &conf->log_opts;
 	slurm_conf_t *cf;
 
-	_update_log = 0;
 	/* Preserve execute line verbose arguments (if any) */
 	cf = slurm_conf_lock();
 	if (log_lvl != LOG_LEVEL_END) {
