@@ -60,9 +60,10 @@
 #include "src/common/spank.h"
 #include "src/common/stepd_api.h"
 #include "src/common/xmalloc.h"
-#include "src/common/xsignal.h"
 #include "src/common/xstring.h"
 
+#include "src/conmgr/conmgr.h"
+
 #include "src/interfaces/acct_gather_energy.h"
 #include "src/interfaces/acct_gather_profile.h"
 #include "src/interfaces/auth.h"
@@ -107,15 +108,6 @@ static void _process_cmdline(int argc, char **argv);
 static pthread_mutex_t cleanup_mutex = PTHREAD_MUTEX_INITIALIZER;
 static bool cleanup = false;
 
-/*
- *  List of signals to block in this process
- */
-int slurmstepd_blocked_signals[] = {
-	SIGINT,  SIGTERM, SIGTSTP,
-	SIGQUIT, SIGPIPE, SIGUSR1,
-	SIGUSR2, SIGALRM, SIGHUP, 0
-};
-
 /* global variable */
 slurmd_conf_t * conf;
 extern char  ** environ;
@@ -219,23 +211,8 @@ static int _foreach_job_node_array(void *x, void *arg)
 	node_record_t *job_node_ptr = x;
 	int *table_index = arg;
 
-	config_record_t *config_ptr;
-	config_ptr = create_config_record();
-	config_ptr->boards = job_node_ptr->boards;
-	config_ptr->core_spec_cnt = job_node_ptr->core_spec_cnt;
-	config_ptr->cores = job_node_ptr->cores;
-	config_ptr->cpu_spec_list = xstrdup(job_node_ptr->cpu_spec_list);
-	config_ptr->cpus = job_node_ptr->cpus;
-	config_ptr->feature = xstrdup(job_node_ptr->features);
-	config_ptr->gres = xstrdup(job_node_ptr->gres);
-	config_ptr->node_bitmap = bit_alloc(node_record_count);
-	config_ptr->nodes = xstrdup(job_node_ptr->name);
-	config_ptr->real_memory = job_node_ptr->real_memory;
-	config_ptr->res_cores_per_gpu = job_node_ptr->res_cores_per_gpu;
-	config_ptr->threads = job_node_ptr->threads;
-	config_ptr->tmp_disk = job_node_ptr->tmp_disk;
-	config_ptr->tot_sockets = job_node_ptr->tot_sockets;
-	config_ptr->weight = job_node_ptr->weight;
+	config_record_t *config_ptr =
+		config_record_from_node_record(job_node_ptr);
 
 	*table_index = bit_ffs_from_bit(job_step_ptr->node_bitmap, *table_index);
 
@@ -296,20 +273,65 @@ static void _init_stepd_stepmgr(void)
 			    NULL);
 }
 
-int
-main (int argc, char **argv)
+static void _on_sigint(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGINT. Shutting down.");
+	conmgr_request_shutdown();
+}
+
+static void _on_sigterm(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGTERM. Shutting down.");
+	conmgr_request_shutdown();
+}
+
+static void _on_sigquit(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGQUIT. Shutting down.");
+	conmgr_request_shutdown();
+}
+
+static void _on_sigtstp(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGTSTP. Ignoring");
+}
+
+static void _on_sighup(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGHUP. Ignoring");
+}
+
+static void _on_sigusr1(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGUSR1. Ignoring.");
+}
+
+static void _on_sigusr2(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGUSR2. Ignoring.");
+}
+
+static void _on_sigpipe(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGPIPE. Ignoring.");
+}
+
+static void _on_sigttin(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	debug("Caught SIGTTIN. Ignoring.");
+}
+
+extern int main(int argc, char **argv)
 {
 	log_options_t lopts = LOG_OPTS_INITIALIZER;
 	slurm_addr_t *cli;
 	slurm_msg_t *msg;
 	stepd_step_rec_t *step;
-	int rc = 0;
+	int rc = SLURM_SUCCESS;
+	bool only_mem = true;
 
 	_process_cmdline(argc, argv);
 
-	run_command_init();
-
-	xsignal_block(slurmstepd_blocked_signals);
 	conf = xmalloc(sizeof(*conf));
 	conf->argv = argv;
 	conf->argc = argc;
@@ -320,6 +342,25 @@ main (int argc, char **argv)
 	/* Receive job parameters from the slurmd */
 	_init_from_slurmd(STDIN_FILENO, argv, &cli, &msg);
 
+	conmgr_init(0, 0, (conmgr_callbacks_t) {0});
+
+	conmgr_add_work_signal(SIGINT, _on_sigint, NULL);
+	conmgr_add_work_signal(SIGTERM, _on_sigterm, NULL);
+	conmgr_add_work_signal(SIGQUIT, _on_sigquit, NULL);
+	conmgr_add_work_signal(SIGTSTP, _on_sigtstp, NULL);
+	conmgr_add_work_signal(SIGHUP, _on_sighup, NULL);
+	conmgr_add_work_signal(SIGUSR1, _on_sigusr1, NULL);
+	conmgr_add_work_signal(SIGUSR2, _on_sigusr2, NULL);
+	conmgr_add_work_signal(SIGPIPE, _on_sigpipe, NULL);
+	conmgr_add_work_signal(SIGTTIN, _on_sigttin, NULL);
+
+	conmgr_run(false);
+
+	if ((run_command_init(argc, argv, conf->stepd_loc) != SLURM_SUCCESS) &&
+	    conf->stepd_loc && conf->stepd_loc[0])
+		fatal("%s: Unable to reliably execute %s",
+		      __func__, conf->stepd_loc);
+
 	/* Create the stepd_step_rec_t, mostly from info in a
 	 * launch_tasks_request_msg_t or a batch_job_launch_msg_t */
 	if (!(step = _step_setup(cli, msg))) {
@@ -364,9 +405,12 @@ main (int argc, char **argv)
 	 * and blocks until the step is complete */
 	rc = job_manager(step);
 
-	return stepd_cleanup(msg, step, cli, rc, false);
+	only_mem = false;
 ending:
-	return stepd_cleanup(msg, step, cli, rc, true);
+	rc = stepd_cleanup(msg, step, cli, rc, only_mem);
+
+	conmgr_fini();
+	return rc;
 }
 
 extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
@@ -408,14 +452,14 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
 	if (step->container)
 		cleanup_container(step);
 
-	run_command_shutdown();
-
 	if (step->step_id.step_id == SLURM_EXTERN_CONT) {
 		if (container_g_stepd_delete(step->step_id.job_id))
 			error("container_g_stepd_delete(%u): %m",
 			      step->step_id.job_id);
 	}
 
+	run_command_shutdown();
+
 	/*
 	 * join() must be done before _step_cleanup() where job_step_ptr is
 	 * freed.
@@ -464,6 +508,9 @@ done:
 	} else {
 		info("done with step");
 	}
+
+	conmgr_request_shutdown();
+
 	return rc;
 }
 
@@ -496,7 +543,7 @@ static slurmd_conf_t *_read_slurmd_conf_lite(int fd)
 	buf_t *buffer = NULL;
 	slurmd_conf_t *confl, *local_conf = NULL;
 	int tmp_int = 0;
-	List tmp_list = NULL;
+	list_t *tmp_list = NULL;
 	assoc_mgr_lock_t locks = { .tres = WRITE_LOCK };
 
 	/*  First check to see if we've already initialized the
@@ -683,14 +730,20 @@ static void _process_cmdline(int argc, char **argv)
 			exit(1);
 		exit(0);
 	}
+	if (run_command_is_launcher(argc, argv)) {
+		run_command_launcher(argc, argv);
+		_exit(127); /* Should not get here */
+	}
 }
 
 static void
 _send_ok_to_slurmd(int sock)
 {
-	/* If running under valgrind/memcheck, this pipe doesn't work correctly
-	 * so just skip it. */
-#if (SLURMSTEPD_MEMCHECK == 0)
+	/*
+	 * If running under memcheck, this pipe doesn't work correctly so just
+	 * skip it.
+	 */
+#if (SLURMSTEPD_MEMCHECK != 1)
 	int ok = SLURM_SUCCESS;
 	safe_write(sock, &ok, sizeof(int));
 	return;
@@ -701,9 +754,11 @@ rwfail:
 
 static void _send_fail_to_slurmd(int sock, int rc)
 {
-	/* If running under valgrind/memcheck, this pipe doesn't work correctly
-	 * so just skip it. */
-#if (SLURMSTEPD_MEMCHECK == 0)
+	/*
+	 * If running under memcheck, this pipe doesn't work correctly so just
+	 * skip it.
+	 */
+#if (SLURMSTEPD_MEMCHECK != 1)
 	safe_write(sock, &rc, sizeof(int));
 	return;
 rwfail:
@@ -714,9 +769,11 @@ rwfail:
 static void
 _got_ack_from_slurmd(int sock)
 {
-	/* If running under valgrind/memcheck, this pipe doesn't work correctly
-	 * so just skip it. */
-#if (SLURMSTEPD_MEMCHECK == 0)
+	/*
+	 * If running under memcheck, this pipe doesn't work correctly so just
+	 * skip it.
+	 */
+#if (SLURMSTEPD_MEMCHECK != 1)
 	int ok;
 	safe_read(sock, &ok, sizeof(int));
 	return;
@@ -896,6 +953,9 @@ _init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
 
 	_set_job_log_prefix(&step_id);
 
+	if (cgroup_read_state(sock) != SLURM_SUCCESS)
+		fatal("Failed to read cgroup state from slurmd");
+
 	/*
 	 * Init all plugins after receiving the slurm.conf from the slurmd.
 	 */
@@ -905,7 +965,7 @@ _init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
 	    (acct_gather_conf_init() != SLURM_SUCCESS) ||
 	    (prep_g_init(NULL) != SLURM_SUCCESS) ||
 	    (proctrack_g_init() != SLURM_SUCCESS) ||
-	    (slurmd_task_init() != SLURM_SUCCESS) ||
+	    (task_g_init() != SLURM_SUCCESS) ||
 	    (jobacct_gather_init() != SLURM_SUCCESS) ||
 	    (acct_gather_profile_init() != SLURM_SUCCESS) ||
 	    (job_container_init() != SLURM_SUCCESS) ||
@@ -1070,25 +1130,13 @@ _step_cleanup(stepd_step_rec_t *step, slurm_msg_t *msg, int rc)
 			stepd_step_rec_destroy(step);
 	}
 
-	if (msg) {
-		/*
-		 * The message cannot be freed until the jobstep is complete
-		 * because the job struct has pointers into the msg, such
-		 * as the switch jobinfo pointer.
-		 */
-		switch(msg->msg_type) {
-		case REQUEST_BATCH_JOB_LAUNCH:
-			slurm_free_job_launch_msg(msg->data);
-			break;
-		case REQUEST_LAUNCH_TASKS:
-			slurm_free_launch_tasks_request_msg(msg->data);
-			break;
-		default:
-			fatal("handle_launch_message: Unrecognized launch RPC");
-			break;
-		}
-		xfree(msg);
-	}
+	/*
+	 * The message cannot be freed until the jobstep is complete
+	 * because the job struct has pointers into the msg, such
+	 * as the switch jobinfo pointer.
+	 */
+	slurm_free_msg(msg);
+
 	jobacctinfo_destroy(step_complete.jobacct);
 }
 #endif
