@@ -46,6 +46,7 @@
 #include <grp.h>
 #include <pthread.h>
 #include <signal.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -58,35 +59,33 @@
 #include "src/common/log.h"
 #include "src/common/proc_args.h"
 #include "src/common/read_config.h"
-#include "src/interfaces/accounting_storage.h"
-#include "src/interfaces/auth.h"
+#include "src/common/sluid.h"
 #include "src/common/slurm_rlimits_info.h"
 #include "src/common/slurm_time.h"
 #include "src/common/uid.h"
 #include "src/common/xmalloc.h"
-#include "src/common/xsignal.h"
 #include "src/common/xstring.h"
 
-#include "src/interfaces/hash.h"
+#include "src/conmgr/conmgr.h"
 
+#include "src/interfaces/accounting_storage.h"
+#include "src/interfaces/auth.h"
+#include "src/interfaces/hash.h"
+#include "src/interfaces/tls.h"
+
+#include "src/slurmdbd/backup.h"
+#include "src/slurmdbd/proc_req.h"
 #include "src/slurmdbd/read_config.h"
 #include "src/slurmdbd/rpc_mgr.h"
-#include "src/slurmdbd/proc_req.h"
-#include "src/slurmdbd/backup.h"
 
 /* Global variables */
 time_t shutdown_time = 0;		/* when shutdown request arrived */
-List registered_clusters = NULL;
+list_t *registered_clusters = NULL;
 pthread_mutex_t rpc_mutex = PTHREAD_MUTEX_INITIALIZER;
 slurmdb_stats_rec_t rpc_stats;
 pthread_mutex_t registered_lock = PTHREAD_MUTEX_INITIALIZER;
-pthread_t signal_handler_thread;	/* thread ID for signal hander */
 
 /* Local variables */
-static int    dbd_sigarray[] = {	/* blocked signals for this process */
-	SIGINT,  SIGTERM, SIGCHLD, SIGUSR1,
-	SIGUSR2, SIGTSTP, SIGXCPU, SIGQUIT,
-	SIGPIPE, SIGALRM, SIGABRT, SIGHUP, 0 };
 static int    debug_level = 0;		/* incremented for -v on command line */
 static bool daemonize = true;		/* run process as a daemon */
 static int    setwd = 0;		/* change working directory -s  */
@@ -101,10 +100,9 @@ static bool running_rollup = 0;
 static bool running_commit = 0;
 static bool restart_backup = false;
 static bool reset_lft_rgt = 0;
-static List lft_rgt_list = NULL;
+static list_t *lft_rgt_list = NULL;
 
 /* Local functions */
-static void  _become_slurm_user(void);
 static void  _commit_handler_cancel(void);
 static void *_commit_handler(void *no_data);
 static void  _daemonize(void);
@@ -119,11 +117,85 @@ static void *_rollup_handler(void *no_data);
 static int   _find_rollup_stats_in_list(void *x, void *key);
 static int   _send_slurmctld_register_req(slurmdb_cluster_rec_t *cluster_rec);
 static void  _set_work_dir(void);
-static void *_signal_handler(void *no_data);
 static void  _update_logging(bool startup);
 static void  _update_nice(void);
 static void  _usage(char *prog_name);
 
+static void _on_sigint(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Terminate signal SIGINT received");
+	shutdown_threads();
+}
+
+static void _on_sigterm(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Terminate signal SIGTERM received");
+	shutdown_threads();
+}
+
+static void _on_sigchld(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGCHLD. Ignoring");
+}
+
+static void _on_sigquit(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Terminate signal SIGQUIT received");
+	shutdown_threads();
+}
+
+static void _on_sigtstp(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGTSTP. Ignoring");
+}
+
+static void _on_sighup(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Reconfigure signal (SIGHUP) received");
+	slurm_thread_create_detached(reconfig, NULL);
+}
+
+static void _on_sigusr1(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	debug("Caught SIGUSR1. Ignoring.");
+}
+
+static void _on_sigusr2(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Logrotate signal (SIGUSR2) received");
+	_update_logging(false);
+}
+
+static void _on_sigpipe(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGPIPE. Ignoring.");
+}
+
+static void _on_sigttin(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	debug("Caught SIGTTIN. Ignoring.");
+}
+
+static void _on_sigxcpu(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	debug("Caught SIGXCPU. Ignoring.");
+}
+
+static void _register_signal_handlers(void)
+{
+	conmgr_add_work_signal(SIGINT, _on_sigint, NULL);
+	conmgr_add_work_signal(SIGTERM, _on_sigterm, NULL);
+	conmgr_add_work_signal(SIGCHLD, _on_sigchld, NULL);
+	conmgr_add_work_signal(SIGQUIT, _on_sigquit, NULL);
+	conmgr_add_work_signal(SIGTSTP, _on_sigtstp, NULL);
+	conmgr_add_work_signal(SIGHUP, _on_sighup, NULL);
+	conmgr_add_work_signal(SIGUSR1, _on_sigusr1, NULL);
+	conmgr_add_work_signal(SIGUSR2, _on_sigusr2, NULL);
+	conmgr_add_work_signal(SIGPIPE, _on_sigpipe, NULL);
+	conmgr_add_work_signal(SIGTTIN, _on_sigttin, NULL);
+	conmgr_add_work_signal(SIGXCPU, _on_sigxcpu, NULL);
+}
+
 /* main - slurmctld main function, start various threads and process RPCs */
 int main(int argc, char **argv)
 {
@@ -137,6 +209,15 @@ int main(int argc, char **argv)
 	if (read_slurmdbd_conf())
 		exit(1);
 	_parse_commandline(argc, argv);
+
+	if (slurmdbd_conf->flags & DBD_CONF_FLAG_GET_DBVER) {
+		become_slurm_user();
+		if (acct_storage_g_init() != SLURM_SUCCESS) {
+			fatal("Unable to initialize %s accounting storage plugin",
+			      slurm_conf.accounting_storage_type);
+		}
+	}
+
 	_update_logging(true);
 	_update_nice();
 
@@ -151,7 +232,11 @@ int main(int argc, char **argv)
 	 * able to write a core dump.
 	 */
 	_init_pidfile();
-	_become_slurm_user();
+	become_slurm_user();
+
+	conmgr_init(0, 0, (conmgr_callbacks_t) {0});
+
+	_register_signal_handlers();
 
 	/*
 	 * Do plugin init's after _init_pidfile so systemd is happy as
@@ -164,11 +249,16 @@ int main(int argc, char **argv)
 	if (hash_g_init() != SLURM_SUCCESS) {
 		fatal("failed to initialize hash plugin");
 	}
+	if (tls_g_init() != SLURM_SUCCESS) {
+		fatal("Failed to initialize tls plugin");
+	}
 	if (acct_storage_g_init() != SLURM_SUCCESS) {
 		fatal("Unable to initialize %s accounting storage plugin",
 		      slurm_conf.accounting_storage_type);
 	}
 
+	sluid_init(0x001, 0);
+
 	if (daemonize || setwd)
 		_set_work_dir();
 	log_config();
@@ -179,12 +269,6 @@ int main(int argc, char **argv)
 		debug ("Unable to set dumpable to 1");
 #endif /* PR_SET_DUMPABLE */
 
-	if (xsignal_block(dbd_sigarray) < 0)
-		error("Unable to block signals");
-
-	/* Create attached thread for signal handling */
-	slurm_thread_create(&signal_handler_thread, _signal_handler, NULL);
-
 	registered_clusters = list_create(NULL);
 
 	slurm_thread_create(&commit_handler_thread, _commit_handler, NULL);
@@ -225,6 +309,8 @@ int main(int argc, char **argv)
 	if (gethostname_short(node_name_short, sizeof(node_name_short)))
 		fatal("getnodename_short: %m");
 
+	conmgr_run(false);
+
 	while (1) {
 		if (slurmdbd_conf->dbd_backup &&
 		    (!xstrcmp(node_name_short, slurmdbd_conf->dbd_backup) ||
@@ -276,14 +362,8 @@ int main(int argc, char **argv)
 		acct_storage_g_commit(db_conn, 1);
 
 		/* this is only ran if not backup */
-		if (rollup_handler_thread) {
-			pthread_join(rollup_handler_thread, NULL);
-			rollup_handler_thread = 0;
-		}
-		if (rpc_handler_thread) {
-			pthread_join(rpc_handler_thread, NULL);
-			rpc_handler_thread = 0;
-		}
+		slurm_thread_join(rollup_handler_thread);
+		slurm_thread_join(rpc_handler_thread);
 
 		if (backup && primary_resumed && !restart_backup) {
 			shutdown_time = 0;
@@ -297,10 +377,7 @@ int main(int argc, char **argv)
 
 end_it:
 
-	if (signal_handler_thread && (!backup || !restart_backup))
-		pthread_join(signal_handler_thread, NULL);
-	if (commit_handler_thread)
-		pthread_join(commit_handler_thread, NULL);
+	slurm_thread_join(commit_handler_thread);
 
 	acct_storage_g_commit(db_conn, 1);
 	acct_storage_g_close_connection(&db_conn);
@@ -324,26 +401,35 @@ end_it:
 	acct_storage_g_fini();
 	auth_g_fini();
 	hash_g_fini();
-	log_fini();
+	tls_g_fini();
 	free_slurmdbd_conf();
 	slurm_mutex_lock(&rpc_mutex);
 	slurmdb_free_stats_rec_members(&rpc_stats);
 	slurm_mutex_unlock(&rpc_mutex);
-	exit(0);
+
+	conmgr_fini();
+	log_fini();
+	return SLURM_SUCCESS;
 }
 
-extern void reconfig(void)
+extern void *reconfig(void *arg)
 {
+	conmgr_quiesce(__func__);
+
 	read_slurmdbd_conf();
 	assoc_mgr_set_missing_uids();
 	acct_storage_g_reconfig(NULL, 0);
 	_update_logging(false);
+
+	conmgr_unquiesce(__func__);
+
+	return NULL;
 }
 
-extern void handle_rollup_stats(List rollup_stats_list,
+extern void handle_rollup_stats(list_t *rollup_stats_list,
 				long delta_time, int type)
 {
-	ListIterator itr;
+	list_itr_t *itr;
 	slurmdb_rollup_stats_t *rollup_stats, *rpc_rollup_stats;
 
 	xassert(type < DBD_ROLLUP_COUNT);
@@ -408,6 +494,8 @@ extern void shutdown_threads(void)
 	_commit_handler_cancel();
 	rpc_mgr_wake();
 	_rollup_handler_cancel();
+
+	conmgr_request_shutdown();
 }
 
 /* Allocate storage for statistics data structure,
@@ -467,7 +555,7 @@ static void _parse_commandline(int argc, char **argv)
 	char *tmp_char;
 
 	opterr = 0;
-	while ((c = getopt(argc, argv, "Dhn:R::svV")) != -1)
+	while ((c = getopt(argc, argv, "Dhn:R::suvV")) != -1)
 		switch (c) {
 		case 'D':
 			daemonize = 0;
@@ -496,6 +584,9 @@ static void _parse_commandline(int argc, char **argv)
 		case 's':
 			setwd = 1;
 			break;
+		case 'u':
+			slurmdbd_conf->flags |= DBD_CONF_FLAG_GET_DBVER;
+			break;
 		case 'v':
 			debug_level++;
 			break;
@@ -530,6 +621,8 @@ static void _usage(char *prog_name)
 		"Change working directory to LogFile dirname or /var/tmp/.\n");
 	fprintf(stderr, "  -v         \t"
 		"Verbose mode. Multiple -v's increase verbosity.\n");
+	fprintf(stderr, "  -u         \t"
+		"Only display the Slurm Database version and if conversion is needed. If no conversion is needed 0 is returned, if conversion is needed 1 is returned.\n");
 	fprintf(stderr, "  -V         \t"
 		"Print version information and exit.\n");
 }
@@ -675,9 +768,9 @@ static void _set_work_dir(void)
 
 static void _request_registrations(void *db_conn)
 {
-	List cluster_list = acct_storage_g_get_clusters(
+	list_t *cluster_list = acct_storage_g_get_clusters(
 		db_conn, getuid(), NULL);
-	ListIterator itr;
+	list_itr_t *itr;
 	slurmdb_cluster_rec_t *cluster_rec = NULL;
 
 	if (!cluster_list)
@@ -697,6 +790,7 @@ static void _request_registrations(void *db_conn)
 
 static void _rollup_handler_cancel(void)
 {
+	slurm_mutex_lock(&rollup_lock);
 	if (running_rollup) {
 		if (backup && running_rollup && primary_resumed)
 			debug("Hard cancelling rollup thread");
@@ -709,11 +803,10 @@ static void _rollup_handler_cancel(void)
 			pthread_cancel(rollup_handler_thread);
 			restart_backup = true;
 		} else {
-			slurm_mutex_lock(&rollup_lock);
 			pthread_cancel(rollup_handler_thread);
-			slurm_mutex_unlock(&rollup_lock);
 		}
 	}
+	slurm_mutex_unlock(&rollup_lock);
 }
 
 static int _find_rollup_stats_in_list(void *x, void *key)
@@ -734,7 +827,7 @@ static void *_rollup_handler(void *db_conn)
 	time_t next_time;
 /* 	int sigarray[] = {SIGUSR1, 0}; */
 	struct tm tm;
-	List rollup_stats_list = NULL;
+	list_t *rollup_stats_list = NULL;
 	DEF_TIMERS;
 
 	(void) pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
@@ -805,7 +898,7 @@ static void _commit_handler_cancel()
 /* _commit_handler - Process commit's of registered clusters */
 static void *_commit_handler(void *db_conn)
 {
-	ListIterator itr;
+	list_itr_t *itr;
 	slurmdbd_conn_t *slurmdbd_conn;
 
 	(void) pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
@@ -876,96 +969,6 @@ static int _send_slurmctld_register_req(slurmdb_cluster_rec_t *cluster_rec)
 	return rc;
 }
 
-/* _signal_handler - Process daemon-wide signals */
-static void *_signal_handler(void *no_data)
-{
-	int rc, sig;
-	int sig_array[] = {SIGINT, SIGTERM, SIGHUP, SIGABRT, SIGUSR2, 0};
-	sigset_t set;
-
-	(void) pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
-	(void) pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
-
-	/* Make sure no required signals are ignored (possibly inherited) */
-	xsignal_default(SIGINT);
-	xsignal_default(SIGTERM);
-	xsignal_default(SIGHUP);
-	xsignal_default(SIGABRT);
-	xsignal_default(SIGUSR2);
-
-	while (1) {
-		xsignal_sigset_create(sig_array, &set);
-		rc = sigwait(&set, &sig);
-		if (rc == EINTR)
-			continue;
-		switch (sig) {
-		case SIGHUP:	/* kill -1 */
-			info("Reconfigure signal (SIGHUP) received");
-			reconfig();
-			break;
-		case SIGINT:	/* kill -2  or <CTRL-C> */
-		case SIGTERM:	/* kill -15 */
-			info("Terminate signal (SIGINT or SIGTERM) received");
-			shutdown_threads();
-			return NULL;	/* Normal termination */
-		case SIGABRT:	/* abort */
-			info("SIGABRT received");
-			abort();	/* Should terminate here */
-			shutdown_threads();
-			return NULL;
-		case SIGUSR2:
-			info("Logrotate signal (SIGUSR2) received");
-			_update_logging(false);
-			break;
-		default:
-			error("Invalid signal (%d) received", sig);
-		}
-	}
-
-}
-
-static void _become_slurm_user(void)
-{
-	gid_t slurm_user_gid;
-
-	/* Determine SlurmUser gid */
-	slurm_user_gid = gid_from_uid(slurm_conf.slurm_user_id);
-	if (slurm_user_gid == (gid_t) -1) {
-		fatal("Failed to determine gid of SlurmUser(%u)",
-		      slurm_conf.slurm_user_id);
-	}
-
-	/* Initialize supplementary groups ID list for SlurmUser */
-	if (getuid() == 0) {
-		/* root does not need supplementary groups */
-		if ((slurm_conf.slurm_user_id == 0) &&
-		    (setgroups(0, NULL) != 0)) {
-			fatal("Failed to drop supplementary groups, "
-			      "setgroups: %m");
-		} else if ((slurm_conf.slurm_user_id != getuid()) &&
-		           initgroups(slurm_conf.slurm_user_name,
-		                      slurm_user_gid)) {
-			fatal("Failed to set supplementary groups, "
-			      "initgroups: %m");
-		}
-	} else {
-		info("Not running as root. Can't drop supplementary groups");
-	}
-
-	/* Set GID to GID of SlurmUser */
-	if ((slurm_user_gid != getegid()) &&
-	    (setgid(slurm_user_gid))) {
-		fatal("Failed to set GID to %u", slurm_user_gid);
-	}
-
-	/* Set UID to UID of SlurmUser */
-	if ((slurm_conf.slurm_user_id != getuid()) &&
-	    (setuid(slurm_conf.slurm_user_id))) {
-		fatal("Can not set uid to SlurmUser(%u): %m",
-		      slurm_conf.slurm_user_id);
-	}
-}
-
 static void _restart_self(int argc, char **argv)
 {
 	info("Restarting self");
